package cn.iocoder.yudao.module.product.component;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.util.bigDecimal.BigDecimalUtils;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPackaging.LineChannelPackagingDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPriceStepClearance.LineChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.lineChannelPriceStepClearance.LineChannelPriceStepClearanceMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.depository.service.lineChannelPackaging.LineChannelPackagingService;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionDto;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionQueryDto;
import cn.iocoder.yudao.module.customer.service.customer.commission.CustomerCommissionInfoService;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.zTest3;
import cn.iocoder.yudao.module.product.dal.dataobject.product.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.product.dal.mysql.coupon.ZTestMapper3;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductAttrMapper;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductPricePackagingMapper;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductPriceStepPackagingMapper;
import cn.iocoder.yudao.module.product.dal.mysql.product.ProductPriceStepSpecialMapper;
import cn.iocoder.yudao.module.product.dto.*;
import cn.iocoder.yudao.module.product.enums.FeeTypeEnum;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceSpecialService;
import cn.iocoder.yudao.module.ecw.service.unit.UnitService;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.service.product.ProductPriceStepService;
import cn.iocoder.yudao.module.product.vo.coupon.CouponInfoVO;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;

/**
 * 商品加个计算
 */
@Component
@AllArgsConstructor
@Slf4j
public class ProdCostCalculation {

    private final ProductPriceService productPriceService;
    private final ProductPriceStepService productPriceStepService;
    private final ProductPriceSpecialService specialService;
    private final ProductPricePackagingMapper pricePackagingMapper;
    private final ProductPriceStepSpecialMapper priceStepSpecialMapper;
    private final ProductPriceStepPackagingMapper priceStepPackagingMapper;
    private final ProductAttrMapper productAttrMapper;
    private final CouponService couponService;
    private final UnitService unitService;
    private final CurrencyApi currencyApi;
    private final CurrencyService currencyService;
    private final WarehouseService warehouseService;
    private final WarehouseLineMapper warehouseLineMapper;
    private final CustomerCommissionInfoService customerCommissionInfoService;
    private final DictDataApi dictDataApi;
    private final LineChannelPackagingService
            lineChannelPackagingService;
    private final LineChannelPriceStepClearanceMapper
            lineChannelPriceStepClearanceMapper;

    //
    @Resource
    private ZTestMapper3 zTestMapper;


    public List<ProdConditionParam> obtainProdLineOnePrice(
            List<ProdConditionParam> prodConditionParamList,
            String no,
            Integer customsType,
            Long customerId,
            Integer transportId,
            Long lineId,
            Long channelId,
            Boolean isWarehouseIn) {

        if (CollectionUtil.isEmpty(prodConditionParamList)) {
            throw exception(PROD_PARAM_NOT_NULL);
        }
        // 获取所有单位
        List<UnitDO> unitDOList = unitService.list();
        Map<Integer, UnitDO> unitDOMap = unitDOList.stream()
                .collect(Collectors.toMap(UnitDO::getId,
                        Function.identity(),
                        (key1, key2) -> key2));

        // 单价查询计算
        long index = 1;
        // 订单填单总重量")
        BigDecimal totalWeight = BigDecimal.ZERO;
        // 订单填单总体积")
        BigDecimal totalVolume = BigDecimal.ZERO;
        // 订单填单总个数")
        BigDecimal totalQuantity = BigDecimal.ZERO;
        // 订单收费总重量")
        BigDecimal chargeTotalWeight = BigDecimal.ZERO;
        // 订单收费总体积")
        BigDecimal chargeTotalVolume = BigDecimal.ZERO;
        // 订单收费总个数")
        BigDecimal chargeTotalQuantity = BigDecimal.ZERO;
        for (ProdConditionParam prodParam : prodConditionParamList) {
            if (Objects.isNull(prodParam.getItemId())) {
                // 商品id可能重复，所以对于没有商品项id的参数以下标做商品项id
                prodParam.setItemId(index++);
            }
            // 查询价格
            this.queryPrice(no,
                    customsType,
                    customerId,
                    transportId,
                    lineId,
                    channelId,
                    unitDOMap, prodParam);

            if (transportId == 3) {
                // 空运的阶梯价格需要计算出订单的所有品名总收费值来匹配价格-->统计品名的总值
                BigDecimal quantity = BigDecimal.valueOf(prodParam.getQuantity());
                totalQuantity = totalQuantity.add(quantity.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? quantity : prodParam.getMinMeteringQuantity());
                totalVolume = totalVolume.add(prodParam.getVolume().compareTo(prodParam.getMinMeteringVolume()) > 0 ? prodParam.getVolume() : prodParam.getMinMeteringVolume());
                totalWeight = totalWeight.add(prodParam.getWeight().compareTo(prodParam.getMinMeteringWeight()) > 0 ? prodParam.getWeight() : prodParam.getMinMeteringWeight());
                if (Objects.nonNull(prodParam.getChargeQuantity())) {
                    chargeTotalQuantity =
                            chargeTotalQuantity.add(prodParam.getChargeQuantity().compareTo(prodParam.getMinMeteringQuantity()) > 0 ? prodParam.getChargeQuantity() : prodParam.getMinMeteringQuantity());
                }
                if (Objects.nonNull(prodParam.getChargeVolume())) {
                    chargeTotalVolume =
                            chargeTotalVolume.add(prodParam.getChargeVolume().compareTo(prodParam.getMinMeteringVolume()) > 0 ? prodParam.getChargeVolume() : prodParam.getMinMeteringVolume());
                }
                if (Objects.nonNull(prodParam.getChargeWeight())) {
                    chargeTotalWeight = chargeTotalWeight.add(prodParam.getChargeWeight().compareTo(prodParam.getMinMeteringWeight()) > 0 ? prodParam.getChargeWeight() : prodParam.getMinMeteringWeight());
                    // 空运收费重量，总重量小于1kg,按1kg算，
                    // 大于1kg的小数点后值，不足0.5按0.5加，
                    // 大于0.5不足1按1.0加，如收费重量3.1=实际收费重量3.5，
                    // 收费重量4.51=实际收费重量5.0
                    if (chargeTotalWeight.compareTo(BigDecimal.ONE) < 0) {
                        chargeTotalWeight = BigDecimal.ONE;
                    } else {
                        chargeTotalWeight = BigDecimalUtils.convertWv(chargeTotalWeight);
                    }
                }
            }
        }
        if (transportId == 3) {
            // 空运的阶梯价格需要计算出订单的所有品名总收费值来匹配价格--> 将总值放入品名列表
            for (ProdConditionParam param : prodConditionParamList) {
                param.setTotalQuantity(totalQuantity);
                param.setTotalVolume(totalVolume);
                param.setTotalWeight(totalWeight);
                param.setChargeTotalQuantity(chargeTotalQuantity);
                param.setChargeTotalVolume(chargeTotalVolume);
                param.setChargeTotalWeight(chargeTotalWeight);
                //param.setOneClearanceFee();
            }
        }
        // 定价计算
        for (ProdConditionParam prodParam : prodConditionParamList) {
            // 品名定价
            this.fixPrice(no,
                    customsType,
                    customerId,
                    transportId,
                    lineId,
                    channelId,
                    unitDOMap,
                    prodParam);

            // 应收运费单价(原始价格(无牌无液无电)+特殊加价+明佣)
            BigDecimal receivableOneSeaFreight =
                    prodParam.getOrgFreightPrice().add(prodParam.getElectrifiedFreightPrice())
                            .add(prodParam.getLiquidFreightPrice()).add(prodParam.getBrandFreightPrice()).add(prodParam.getPlainCommission());
            prodParam.setReceivableOneSeaFreight(receivableOneSeaFreight);
            // 应收运费
            this.setReceivableTransportPrice(unitDOMap, prodParam);
            if (prodParam.getCharging() == 0) {
                // 应收清关费
                // 应收清关费单价(原始价格(无牌无液无电)+特殊加价)")
                BigDecimal receivableOneClearanceFreight =
                        prodParam.getOrgClearanceFeePrice().add(
                                        prodParam.getElectrifiedClearanceFeePrice())
                                .add(prodParam.getLiquidClearanceFeePrice()).
                                add(prodParam.getBrandClearanceFeePrice());
                prodParam.setReceivableOneClearanceFreight(
                        receivableOneClearanceFreight);
                this.setReceivableClearancePrice(unitDOMap,
                        prodParam, transportId);
            } else {
                // 全价计算，都以运费计算完成，不再计算清关费
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneClearanceFee(BigDecimal.ZERO);
                    prodParam.setReceivableOneClearanceFreight(BigDecimal.ZERO);
                }
                prodParam.setReceivableClearanceFreight(
                        prodParam.getReceivableOneClearanceFreight());
            }
        }
        return prodConditionParamList;
    }

    /**
     * 计算单种运输方式的商品费用
     *
     * @param prodConditionParamList      商品条件列表
     * @param no                          业务编号
     * @param orderType                   订单属性： 1 普货  2 重货 3 泡货
     * @param customsType                 报关方式: 我司全代：1，自单代报：2，混合报关：3
     * @param isControl                   是否控货： 0 否  1 是
     * @param userId                      用户ID
     * @param customerId                  客户ID
     * @param consignorCustomerId         发货客户ID
     * @param consigneeCustomerId         收货客户ID
     * @param consignorCustomerContactsId 发货客户联系人ID
     * @param consigneeCustomerContactsId 收货客户联系人ID
     * @param transportId                 运输方式
     * @param lineId                      线路ID
     * @param channelId                   渠道ID
     * @return 商品列表计费后信息
     */
    public CalculationCostResultDto calculationProdCost(List<ProdConditionParam> prodConditionParamList,
                                                        String no,
                                                        Integer orderType,
                                                        Integer customsType,
                                                        Integer isControl,
                                                        Long userId,
                                                        Long customerId,
                                                        Long consignorCustomerId,
                                                        Long consigneeCustomerId,
                                                        Long consignorCustomerContactsId,
                                                        Long consigneeCustomerContactsId,
                                                        Integer transportId,
                                                        Long lineId, Long channelId) {
        //获取所有货币
        List<CurrencyDO> currencyDOList = currencyService.list();
        if (CollectionUtil.isEmpty(currencyDOList)) {
            throw exception(NOT_FIND_CURRENCY);
        }

        // 美元货币ID 1
        Integer usdCurrency =
                currencyDOList.stream().filter(currencyDO -> StringUtils.equals("$",
                        currencyDO.getFuhao())).findFirst().get().getId();
        // 人民币货币ID 3
        Integer rmbCurrency =
                currencyDOList.stream().filter(currencyDO -> StringUtils.equals("￥",
                        currencyDO.getFuhao())).findFirst().get().getId();

        if (CollectionUtil.isEmpty(prodConditionParamList)) {
            throw exception(PROD_PARAM_NOT_NULL);
        }
//        // 获取所有单位
        List<UnitDO> unitDOList = unitService.list();
        Map<Integer, UnitDO> unitDOMap = unitDOList.stream()
                .collect(Collectors.toMap(UnitDO::getId,
                        Function.identity(), (key1, key2) -> key2));

        //prodCostDtoList 计价返回值
        List<ProdCostDto> prodCostDtoList = new ArrayList<>();
        // 统计箱规
        String l = "0";
        String w = "0";
        String h = "0";
        List<CouponAvailableGroupDto> couponAvailableGroupDtoList = new ArrayList<>();

        couponService.getCouponUserAvailableDto(transportId,
                orderType, customsType, isControl,
                userId, consignorCustomerContactsId,
                consigneeCustomerContactsId,
                lineId, channelId,
                prodConditionParamList,
                rmbCurrency, unitDOMap);
        // 计算订单费用
        for (ProdConditionParam prodParam : prodConditionParamList) {
            // 优惠金额：优惠活动减免
            // 优惠计算
            CouponAvailableGroupDto groupDto = prodParam.getGroupDto();
            this.discountCalculation(prodParam, groupDto);
            if (StringUtils.isNotBlank(prodParam.getBoxGauge())) {
                String[] prodBox = prodParam.getBoxGauge().split("\\*");
                if (prodBox.length == 3) {
                    String prodL = prodBox[0];
                    String prodW = prodBox[1];
                    String prodH = prodBox[2];
                    l = new BigDecimal(l).add(new BigDecimal(prodL)).toString();
                    w = new BigDecimal(w).add(new BigDecimal(prodW)).toString();
                    h = new BigDecimal(h).add(new BigDecimal(prodH)).toString();
                }
            }
            // 运费
            this.setTransportPrice(unitDOMap, prodParam);

            if (prodParam.getCharging() == 0) {
                // 全价计算，都以运费计算完成，不再计算清关费
                // 清关费
                this.setClearancePrice(unitDOMap, prodParam, transportId);
            } else {
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneClearanceFee(BigDecimal.ZERO);
                }
                prodParam.setClearanceFee(prodParam.getOneClearanceFee());
                BigDecimal notDiscountClearanceFeePrice = prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice());
                prodParam.setDiscountClearanceFeePrice(notDiscountClearanceFeePrice.subtract(prodParam.getClearanceFee()));
            }


            // 优惠列表封装优惠总额
            if (Objects.nonNull(groupDto) && CollectionUtil.isNotEmpty(groupDto.getCouponAvailableDtoList())) {
                CouponAvailableDto couponAvailableDto = groupDto.getCouponAvailableDtoList().get(0);
                if (Objects.nonNull(couponAvailableDto)) {
                    if (couponAvailableDto.getCostType() == 1) {
                        couponAvailableDto.setReduceTotalAmount(prodParam.getDiscountFreightTotalPrice());
                    }
                    if (couponAvailableDto.getCostType() == 2) {
                        couponAvailableDto.setClearanceReduceAmount(prodParam.getDiscountClearanceFeeTotalPrice());
                    }
                }
                groupDto.setCouponAvailableDtoList(Collections.singletonList(couponAvailableDto));
                couponAvailableGroupDtoList.add(groupDto);
            }

            if (Objects.isNull(prodParam.getWorth())) {
                prodParam.setWorth(BigDecimal.ZERO);
            }
            if (Objects.isNull(prodParam.getWorthCurrency())) {
                prodParam.setWorthCurrency(rmbCurrency);
            }
            this.costEncapsulation(prodCostDtoList, prodParam, groupDto);
        }

        String totalBoxGauge = l + "*" + w + "*" + h;
        CalculationCostResultDto dto = new CalculationCostResultDto();
        List<FeeDto> feeDtoList = new ArrayList<>();
        List<FeeDto> freightDtoList = prodCostDtoList.stream()
                .collect(Collectors.groupingBy(ProdCostDto::getFreightCurrency, CollectorsUtil.summingBigDecimal(ProdCostDto::getFreight)))
                .entrySet()
                .stream().map(kv -> {
                    if (kv.getValue().compareTo(BigDecimal.ZERO) > 0) {
                        FeeDto feeDto = new FeeDto();
                        feeDto.setFeeType(FeeTypeEnum.FREIGHT.getType());
                        feeDto.setAmount(kv.getValue().setScale(0, RoundingMode.HALF_UP));
                        feeDto.setCurrencyId(kv.getKey());
                        return feeDto;
                    }
                    return null;
                }).filter(Objects::nonNull).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(freightDtoList)) {
            feeDtoList.addAll(freightDtoList);
        }

        List<FeeDto> clearanceDtoList = prodCostDtoList.stream().collect(Collectors.groupingBy(ProdCostDto::getClearanceFeeCurrency, CollectorsUtil.summingBigDecimal(ProdCostDto::getClearanceFee)))
                .entrySet()
                .stream().map(kv -> {
                    if (kv.getValue().compareTo(BigDecimal.ZERO) > 0) {
                        FeeDto feeDto = new FeeDto();
                        feeDto.setFeeType(FeeTypeEnum.CLEARANCE_FEE.getType());
                        feeDto.setAmount(kv.getValue().setScale(0, RoundingMode.HALF_UP));
                        feeDto.setCurrencyId(kv.getKey());
                        return feeDto;
                    }
                    return null;
                }).filter(Objects::nonNull).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(clearanceDtoList)) {
            feeDtoList.addAll(clearanceDtoList);
        }

        CostDto costDto = new CostDto();

        costDto.setTotalBoxGauge(totalBoxGauge);
        Integer totalNum = prodCostDtoList.stream().peek(p -> {
            if (null == p.getNum()) {
                p.setNum(0);
            }
        }).mapToInt(ProdCostDto::getNum).sum();
        costDto.setTotalNum(totalNum);
        Integer totalQuantity = prodCostDtoList.stream().peek(p -> {
            if (null == p.getQuantity()) {
                p.setQuantity(0);
            }
        }).mapToInt(ProdCostDto::getQuantity).sum();
        costDto.setTotalQuantity(totalQuantity);
        BigDecimal totalWorth = prodCostDtoList.stream().collect(CollectorsUtil.summingBigDecimal(ProdCostDto::getWorth));
        costDto.setTotalWorth(totalWorth);
        BigDecimal totalVolume = prodCostDtoList.stream().collect(CollectorsUtil.summingBigDecimal(ProdCostDto::getVolume));
        costDto.setTotalVolume(totalVolume);
        BigDecimal totalWeight = prodCostDtoList.stream().collect(CollectorsUtil.summingBigDecimal(ProdCostDto::getWeight));
        costDto.setTotalWeight(totalWeight);
        dto.setProdCostDtoList(prodCostDtoList);
        // 计算报价费用 货值：与保价费相关：保价费 = 货值*1.1*0.002*汇率，
        // 汇率为人民币换目标货币(默认为美元)的汇率
        Long insuranceCurrencyId =
                Objects.nonNull(usdCurrency) ? usdCurrency.longValue() : null;
        BigDecimal minInsuranceFee = new BigDecimal("2");// 2美元的最低保费
        WarehouseLineDO warehouseLineDO = warehouseLineMapper.selectById(lineId);
        if (Objects.nonNull(warehouseLineDO) &&
                Objects.nonNull(warehouseLineDO.getCurrencyUnit())
                && warehouseLineDO.getCurrencyUnit() > 0) {

            //当线路服务中设置了保费货币单位时，就以服务设置的为准，否则使用默认美元
            insuranceCurrencyId = warehouseLineDO.getCurrencyUnit();
            minInsuranceFee = warehouseLineDO.getMinPrice();
        }
        if (Objects.nonNull(insuranceCurrencyId)) {
            ExchangeRateRespDTO exchangeRateRespDTO =
                    currencyApi.getCurrencyRate(rmbCurrency.longValue(),
                            insuranceCurrencyId);
            FeeDto feeDto = new FeeDto();
            if (costDto.getTotalWorth().compareTo(BigDecimal.ZERO) > 0) {
                // 因为货值为RMB计算，保价费应换算为目标货币(默认为美元)
                BigDecimal insuranceFee = costDto.getTotalWorth().multiply(new BigDecimal("1.1"))
                        .multiply(new BigDecimal("0.002")).multiply(exchangeRateRespDTO.getCurrencyRate()).setScale(0, RoundingMode.HALF_UP);
                costDto.setInsuranceFee(insuranceFee);
                feeDto.setAmount(insuranceFee);
            } else {
                costDto.setInsuranceFee(minInsuranceFee);
                feeDto.setAmount(minInsuranceFee);
            }
            if (costDto.getInsuranceFee().compareTo(minInsuranceFee) < 0) {
                costDto.setInsuranceFee(minInsuranceFee); // 保价费最低值
                feeDto.setAmount(minInsuranceFee);
            } else {
                feeDto.setAmount(costDto.getInsuranceFee());
            }
            costDto.setInsuranceCurrencyId(insuranceCurrencyId);
            feeDto.setCurrencyId(insuranceCurrencyId.intValue());
            feeDto.setFeeType(FeeTypeEnum.INSURANCE_FEE.getType());
            feeDtoList.add(feeDto);
        }

        FeeDto feeDto = new FeeDto();
        feeDto.setFeeType(FeeTypeEnum.WORTH.getType());
        feeDto.setAmount(costDto.getTotalWorth());
        feeDto.setCurrencyId(rmbCurrency);
        feeDtoList.add(feeDto);
        //TODO 是否计算总重量与体积
        costDto.setFeeDtoList(feeDtoList);
        dto.setCostDto(costDto);
        if (CollectionUtil.isNotEmpty(couponAvailableGroupDtoList)) {
            CouponUserAvailableDto availableDto = new CouponUserAvailableDto();
            availableDto.setCouponAvailableGroupDtoList(couponAvailableGroupDtoList);
            dto.setAvailableDto(availableDto);
        }
        return dto;
    }

    /**
     * 品名费用封装
     *
     * @param prodCostDtoList 品名费用封装列表
     * @param prodParam       品名
     * @param groupDto        优惠信息
     */
    private void costEncapsulation(List<ProdCostDto> prodCostDtoList,
                                   ProdConditionParam prodParam,
                                   CouponAvailableGroupDto groupDto) {
        ProdCostDto prodCostDto = new ProdCostDto();
        BeanUtils.copyProperties(prodParam, prodCostDto);
        prodCostDto.setProdId(prodParam.getProdId());
        prodCostDto.setWorth(prodParam.getWorth());
        prodCostDto.setWorthCurrency(prodParam.getWorthCurrency());
        prodCostDto.setGroupDto(groupDto);

        prodCostDto.setMinMeteringQuantity(prodParam.getMinMeteringQuantity());
        prodCostDto.setMinMeteringVolume(prodParam.getMinMeteringVolume());
        prodCostDto.setMinMeteringWeight(prodParam.getMinMeteringWeight());

        prodCostDto.setSpecialCommission(prodParam.getSpecialCommission());
        prodCostDto.setLiquidFreightPrice(prodParam.getLiquidFreightPrice());
        prodCostDto.setLiquidClearanceFeePrice(prodParam.getLiquidClearanceFeePrice());
        prodCostDto.setElectrifiedFreightPrice(prodParam.getElectrifiedFreightPrice());
        prodCostDto.setElectrifiedClearanceFeePrice(prodParam.getElectrifiedClearanceFeePrice());
        prodCostDto.setBrandFreightPrice(prodParam.getBrandFreightPrice());
        prodCostDto.setBrandClearanceFeePrice(prodParam.getBrandClearanceFeePrice());
        prodCostDto.setPackagingFreightPrice(prodParam.getPackagingFreightPrice());
        prodCostDto.setPackagingClearanceFeePrice(prodParam.getPackagingClearanceFeePrice());
        prodCostDto.setCommissionType(prodParam.getCommissionType());
        prodCostDto.setPlainCommission(prodParam.getPlainCommission());
        prodCostDto.setHiddenCommission(prodParam.getHiddenCommission());
        prodCostDto.setOneClearanceFee(prodParam.getOneClearanceFee());
        prodCostDto.setOrgClearanceFeePrice(prodParam.getOrgClearanceFeePrice());
        //清关费
        prodCostDto.setClearanceFee(prodParam.getClearanceFee());
        prodCostDto.setProductPriceEmptyType(prodParam.getProductPriceEmptyType());
        prodCostDto.setChannelPriceEmptyType(prodParam.getChannelPriceEmptyType());
        prodCostDto.setIsAirWeightLimit(prodParam.getIsAirWeightLimit());
        prodCostDto.setReceivableClearanceFreight(prodParam.getReceivableClearanceFreight());
        prodCostDto.setDiscountClearanceFeePrice(prodParam.getDiscountClearanceFeePrice());
        prodCostDto.setDiscountClearanceFeeTotalPrice(prodParam.getDiscountClearanceFeeTotalPrice());
        prodCostDto.setCouponInfoVOList(prodParam.getCouponInfoVOList());
        prodCostDto.setAirClearanceSource(prodParam.getAirClearanceSource());
        prodCostDto.setAirClearanceInfo(prodParam.getAirClearanceInfo());
        prodCostDto.setClearanceFeeVolume(prodParam.getClearanceFeeVolume());
        prodCostDto.setClearanceFeeCurrency(prodParam.getClearanceFeeCurrency());
        prodCostDto.setOneFreight(prodParam.getOneFreight());
        prodCostDto.setOrgFreightPrice(prodParam.getOrgFreightPrice());
        //运费
        prodCostDto.setFreight(prodParam.getFreight());
        prodCostDto.setReceivableSeaFreight(prodParam.getReceivableSeaFreight());
        prodCostDto.setDiscountFreightPrice(prodParam.getDiscountFreightPrice());
        prodCostDto.setDiscountFreightTotalPrice(prodParam.getDiscountFreightTotalPrice());
        prodCostDto.setFreightVolume(prodParam.getFreightVolume());
        prodCostDto.setFreightCurrency(prodParam.getFreightCurrency());
        prodCostDto.setIsPayAdvance(prodParam.getIsPayAdvance());
        prodCostDto.setCalculationType(prodParam.getCalculationType());
        prodCostDto.setCharging(prodParam.getCharging());
        prodCostDto.setNeedOrderInquiry(prodParam.getNeedOrderInquiry());
        prodCostDto.setItemId(prodParam.getItemId());

        prodCostDtoList.add(prodCostDto);
    }

    /**
     * 优惠计算
     *
     * @param prodParam 商品信息
     * @param groupDto  可用优惠信息
     */
    private void discountCalculation(ProdConditionParam prodParam,
                                     CouponAvailableGroupDto groupDto) {
        // 没有定价或者指定更新订单品名优惠，并且成交单价大于0时，去实时更新品名的优惠信息
        if ((Objects.isNull(prodParam.getIsPriced()) || !prodParam.getIsPriced() || prodParam.getCalculationType() == 6) && !prodParam.getSplitCustomPriceType() && !prodParam.getSpecialPriceType() && !prodParam.getChannelManualPricing() && prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
            BigDecimal orgFreightReduceAmount = BigDecimal.ZERO;
            BigDecimal orgClearanceReduceAmount = BigDecimal.ZERO;
            if (Objects.nonNull(groupDto) && CollectionUtil.isNotEmpty(groupDto.getCouponAvailableDtoList())) {
                groupDto.setProdTitleZh(prodParam.getProdTitleZh());
                groupDto.setProdTitleEn(prodParam.getProdTitleEn());
                BigDecimal freightReduceAmount = BigDecimal.ZERO;
                BigDecimal clearanceReduceAmount = BigDecimal.ZERO;
                List<CouponInfoVO> couponInfoVOList = new ArrayList<>();
                if (Objects.nonNull(prodParam.getIsPriced()) && prodParam.getIsPriced() && prodParam.getCalculationType() == 6) {
                    // 当是已定价的产品，并且执行活动优惠重算时，计算方式为 新成交单价=原成交单价+原优惠金额-当前优惠金额
                    orgFreightReduceAmount = prodParam.getDiscountFreightPrice();
                    orgClearanceReduceAmount = prodParam.getDiscountClearanceFeePrice();
                }
                for (CouponAvailableDto availableDto : groupDto.getCouponAvailableDtoList()) {
                    if (Objects.nonNull(availableDto)) {
                        freightReduceAmount = freightReduceAmount.add(Objects.nonNull(availableDto.getReduceAmount()) ? availableDto.getReduceAmount() : BigDecimal.ZERO);
                        clearanceReduceAmount = clearanceReduceAmount.add(Objects.nonNull(availableDto.getClearanceReduceAmount()) ? availableDto.getClearanceReduceAmount() : BigDecimal.ZERO);
                        // 使用的优惠活动详情列表
                        couponInfoVOList.addAll(availableDto.getCouponInfoVOList());

                    }
                }
                prodParam.setDiscountFreightPrice(freightReduceAmount);
                prodParam.setDiscountClearanceFeePrice(clearanceReduceAmount);
                prodParam.setCouponInfoVOList(couponInfoVOList);
            } else {
                prodParam.setDiscountFreightPrice(BigDecimal.ZERO);
                prodParam.setDiscountClearanceFeePrice(BigDecimal.ZERO);
                prodParam.setCouponInfoVOList(new ArrayList<>());
            }
            // 成交价 = 初始价+特性加价+明佣-优惠金额
            // 初始价+特性加价 - 优惠
            if (prodParam.getOrgFreightPrice().compareTo(BigDecimal.ZERO) > 0) {
                if (Objects.isNull(prodParam.getDiscountFreightPrice())) {
                    prodParam.setDiscountFreightPrice(BigDecimal.ZERO);
                }
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(orgFreightReduceAmount).subtract(prodParam.getDiscountFreightPrice()));
                }
            }
            if (prodParam.getOrgClearanceFeePrice().compareTo(BigDecimal.ZERO) > 0) {
                if (Objects.isNull(prodParam.getDiscountClearanceFeePrice())) {
                    prodParam.setDiscountClearanceFeePrice(BigDecimal.ZERO);
                }
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneClearanceFee(
                            prodParam.getOneClearanceFee().
                                    add(orgClearanceReduceAmount).
                                    subtract(prodParam.getDiscountClearanceFeePrice()));
                }
            }
        }
    }

    private void queryPrice(String no,
                            Integer customsType,
                            Long customerId,
                            Integer transportId,
                            Long lineId,
                            Long channelId,
                            Map<Integer, UnitDO> unitDOMap,
                            ProdConditionParam prodParam) {
        if (null == prodParam.getWorth()) {
            prodParam.setWorth(BigDecimal.ZERO);
        }
        if (null == prodParam.getVolume()) {
            prodParam.setVolume(BigDecimal.ZERO);
        }
        if (null == prodParam.getWeight()) {
            prodParam.setWeight(BigDecimal.ZERO);
        }
        if (null == prodParam.getQuantity()) {
            prodParam.setQuantity(0);
        }
        if (null == prodParam.getNum()) {
            prodParam.setNum(0);
        }
        if (Objects.nonNull(prodParam.getSpecialPriceType()) &&
                prodParam.getSpecialPriceType()) {
            if (Objects.isNull(prodParam.getOneFreight())) {
                throw exception(SPECIAL_PRICE_ONE_FREIGHT_NOT_NULL);
            }
            if (prodParam.getCharging() == 0) {
                if (Objects.isNull(prodParam.getOneClearanceFee())) {
                    throw exception(SPECIAL_PRICE_ONE_CLEARANCE_FEE_NOT_NULL);
                }
            }
        }

        if ((Objects.isNull(prodParam.getIsPriced()) ||
                !prodParam.getIsPriced() || prodParam.getIsChangeFeeValue())
                && !prodParam.getSplitCustomPriceType() &&
                !prodParam.getSpecialPriceType() &&
                !prodParam.getChannelManualPricing()) {

            //根据商品ID与线路ID获取商品运费与清关费的单价
            ProductPriceDO productPriceDO =
                    productPriceService.getOne(new LambdaQueryWrapper<ProductPriceDO>()
                            .eq(ProductPriceDO::getProductId, prodParam.getProdId())
                            .eq(ProductPriceDO::getWarehouseLineId, lineId)
                            .and(we -> we.le(ProductPriceDO::getValidateStartDate, DateUtil.date())
                                    .or().isNull(ProductPriceDO::getValidateStartDate))
                            .and(we -> we.ge(ProductPriceDO::getValidateEndDate, DateUtil.date())
                                    .or().isNull(ProductPriceDO::getValidateEndDate))
                            .eq(Objects.nonNull(transportId) && (3 == transportId || 4 == transportId) &&
                                    null != channelId && 0 != channelId, ProductPriceDO::getShippingChannelId, channelId)
                            .eq(ProductPriceDO::getDeleted, false)
                            .last("limit 1"));

            if (Objects.nonNull(productPriceDO)) {
                if (Objects.nonNull(prodParam.getIsPriced()) &&
                        prodParam.getIsPriced()) {

                    if (productPriceDO.getStepPrice() == 0 &&
                            prodParam.getIsChangeFeeValue()) {
                        return;
                    }
                }
                if (StringUtils.isNotBlank(no)) {
                    if (no.startsWith("2")) {
                        log.info("===== 报价单【{}】 线路无牌无液无电价：{} =====",
                                no, JSONObject.toJSONString(productPriceDO));
                    } else {
                        log.info("===== 订单【{}】 线路无牌无液无电价：{} =====",
                                no, JSONObject.toJSONString(productPriceDO));
                    }
                } else {
                    log.info("===== 线路无牌无液无电价：{} =====",
                            JSONObject.toJSONString(productPriceDO));
                }
                if (productPriceDO.getBlacklist() == 1 ||
                        productPriceDO.getAuditStatus() == 1) {

                    String lineName = I18nMessage.getLang() == 1 ? "unknown to unknown" : "未知到未知";
                    LogisticsInfoDto logisticsInfoDto =
                            warehouseLineMapper.getStartInfoAndDestInfoByLineId(lineId);
                    if (Objects.nonNull(logisticsInfoDto)) {
                        String startTitle = I18nMessage.getLang() == 1 ?
                                StringUtils.isNotBlank(logisticsInfoDto.getStartTitleEn()) ? logisticsInfoDto.getStartTitleEn() : "unknown" :
                                StringUtils.isNotBlank(logisticsInfoDto.getStartTitleZh()) ? logisticsInfoDto.getStartTitleZh() : "未知";
                        String destTitle = I18nMessage.getLang() == 1 ?
                                StringUtils.isNotBlank(logisticsInfoDto.getDestTitleEn()) ? logisticsInfoDto.getDestTitleEn() : "unknown" :
                                StringUtils.isNotBlank(logisticsInfoDto.getDestTitleZh()) ? logisticsInfoDto.getDestTitleZh() : "未知";

                        lineName = startTitle + "到" + destTitle;
                    }
                    if (productPriceDO.getBlacklist() == 1) {
                        // 黑名单产品不允许添加
                        throw exception(THIS_LINE_PROD_ON_BLACK_LIST,
                                I18nMessage.getLang() == 1 ? prodParam.getProdTitleEn() : prodParam.getProdTitleZh(), lineName);
                    } else {
                        // 下架的线路产品不允许添加
                        throw exception(THIS_LINE_PROD_OFF_SHELF,
                                I18nMessage.getLang() == 1 ? prodParam.getProdTitleEn() : prodParam.getProdTitleZh(), lineName);
                    }
                }

                if (productPriceDO.getNeedBook() == 1) {
                    // 预约入仓，控制上限
                    if (productPriceDO.getDayLimit().subtract(prodParam.getAlreadyExistsTotalVolume().add(prodParam.getVolume())).compareTo(BigDecimal.ZERO) < 0) {
                        throw exception(PROD_VOLUME_BEYOND_DAY_UPPER_LIMIT, I18nMessage.getLang() == 0 ? prodParam.getProdTitleZh() : prodParam.getProdTitleEn());
                    }
                }
                // 线路价格的单询状态
                prodParam.setNeedOrderInquiry(productPriceDO.getNeedOrderInquiry());
                // 确定计量单位
                prodParam.setFreightVolume(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportVolumeUnit() : productPriceDO.getAllVolumeUnit());
                UnitDO unitDO = unitDOMap.get(
                        Integer.valueOf(prodParam.getFreightVolume().toString()));
                if (unitDO != null) {
                    if (StringUtils.equals("m³", unitDO.getFuhao())) {
                        // 体积
                        prodParam.setMinMeteringVolume(productPriceDO.getMinWeight());
                        prodParam.setMinMeteringWeight(BigDecimal.ZERO);
                        prodParam.setMinMeteringQuantity(BigDecimal.ZERO);
                    } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
                        // 重量
                        prodParam.setMinMeteringVolume(BigDecimal.ZERO);
                        prodParam.setMinMeteringWeight(productPriceDO.getMinWeight());
                        prodParam.setMinMeteringQuantity(BigDecimal.ZERO);
                    } else if (StringUtils.equals("a", unitDO.getFuhao())) {
                        // 数量
                        prodParam.setMinMeteringQuantity(productPriceDO.getMinWeight());
                        prodParam.setMinMeteringQuantity(BigDecimal.ZERO);
                        prodParam.setMinMeteringQuantity(BigDecimal.ZERO);
                    } else {
                        throw exception(UNIT_NOT_LEGAL);
                    }
                    prodParam.setUnitDO(unitDO);
                }
                if (productPriceDO.getPriceType() == 0 &&
                        Objects.nonNull(productPriceDO.getClearanceVolumeUnit())) {

                    prodParam.setClearanceFeeVolume(
                            productPriceDO.getClearanceVolumeUnit());
                    UnitDO clearanceUnitDO =
                            unitDOMap.get(Integer.valueOf(prodParam.getClearanceFeeVolume().toString()));
                    prodParam.setClearanceUnitDO(clearanceUnitDO);
                }
                prodParam.setProductPriceDO(productPriceDO);
            }
        }
    }

    /**
     * 定价业务
     *
     * @param no          业务编号
     * @param customsType 报关类型
     * @param customerId  客户id
     * @param transportId 运输方式
     * @param lineId      线路id
     * @param channelId   渠道id
     * @param unitDOMap   计量单位map
     * @param prodParam   品名信息
     */
    private void fixPrice(String no,
                          Integer customsType,
                          Long customerId,
                          Integer transportId,
                          Long lineId,
                          Long channelId,
                          Map<Integer, UnitDO> unitDOMap,
                          ProdConditionParam prodParam) {

        BigDecimal transportPrice = BigDecimal.ZERO;// 初始化商品单价运费
        BigDecimal clearancePrice = BigDecimal.ZERO;// 初始化商品单价清关费
        //是否拆单自定义价格 getIsPriced
        //是否特价 getSpecialPriceType
        //是否为渠道/单询手动定价 getChannelManualPricing
        //是否已定价  getIsPriced
        if ((Objects.isNull(prodParam.getIsPriced()) || !prodParam.getIsPriced())
                && !prodParam.getSplitCustomPriceType() &&
                !prodParam.getSpecialPriceType() &&
                !prodParam.getChannelManualPricing()) {

            // 根据商品ID与线路ID获取商品运费与清关费的单价
            ProductPriceDO productPriceDO = prodParam.getProductPriceDO();
            if (Objects.nonNull(productPriceDO)) {
                if (productPriceDO.getStepPrice() == 1) {
                    // 阶梯价格查询
                    if (transportId == 1 || transportId == 2) {
                        //海运拼柜
                        this.stepSelect(unitDOMap, prodParam, productPriceDO);
                    } else {
                        //专线空运
                        zTest3 z = new zTest3();
                        z.setCreatedate(new Date());
                        String s = "1开始计算空运清关价 airStepSelect,lineId=" + lineId +
                                " 渠道channelId=" + channelId;
                        z.setTestname(s);
                        zTestMapper.insert(z);
                        //空运阶梯价计算
                        this.airStepSelect(unitDOMap, prodParam,
                                productPriceDO, lineId, channelId);
                    }
                } else {
                    zTest3 z = new zTest3();
                    z.setCreatedate(new Date());
                    String s = "不是阶梯价 airStepSelect,lineId=" + lineId +
                            " 渠道channelId=" + channelId;
                    z.setTestname(s);
                    zTestMapper.insert(z);
                }

                prodParam.setIsPayAdvance(productPriceDO.getNeedPay());
                prodParam.setCharging(productPriceDO.getPriceType());

                // 初始价：线路无牌无液无电价
                // 运费
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
                }
                prodParam.setOrgFreightPrice(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
                prodParam.setFreightCurrency(Math.toIntExact(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPriceUnit() : productPriceDO.getAllPriceUnit()));
                // 清关费
                if (productPriceDO.getPriceType() == 0) {
                    //如果是运费加清关费
                    if (!prodParam.getSpecialPriceType()) {
                        //不是特价
                        prodParam.setOneClearanceFee(
                                productPriceDO.getClearancePrice());
                    }
                    prodParam.setOrgClearanceFeePrice(
                            productPriceDO.getClearancePrice());
                    prodParam.setClearanceFeeCurrency(
                            Math.toIntExact(productPriceDO.getClearancePriceUnit()));
                    prodParam.setClearanceFeeVolume(
                            productPriceDO.getClearanceVolumeUnit());
                    if (transportId == 3) {
                        prodParam.setProductPriceEmptyType(
                                productPriceDO.getProductPriceEmptyType());
                    }
                } else {
                    prodParam.setClearanceFee(BigDecimal.ZERO);
                    prodParam.setOneClearanceFee(BigDecimal.ZERO);
                    prodParam.setOrgClearanceFeePrice(BigDecimal.ZERO);
                    prodParam.setClearanceFeeVolume(0L);
                    prodParam.setClearanceFeeCurrency(0);
                }

                if (prodParam.getOrgFreightPrice().compareTo(BigDecimal.ZERO) > 0) {
                    Map<String, List<ProductAttrDO>> attrDOMap = null;
                    if (StringUtils.isNotBlank(prodParam.getProdAttrIds())) {
                        // 特性不为空时执行
                        // 获取该商品所有特性
                        List<ProductAttrDO> attrDOList = productAttrMapper.selectBatchIds(Arrays.asList(prodParam.getProdAttrIds().split(StrUtil.COMMA)));
                        if (CollectionUtil.isNotEmpty(attrDOList)) {
                            attrDOMap = attrDOList.stream().collect(Collectors.groupingBy(ProductAttrDO::getAttrCode));
                        }
                    }
                    // 特性加价：有牌+有液+带电+中性加价
                    // 包装加价
                    // 获取路线商品特殊价格需求和指定包装类型的包装加价
                    if (productPriceDO.getStepPrice() == 1) {
                        // TODO 指定包装类型的包装加价
                        //包装类型
                        if (CollectionUtil.isNotEmpty(prodParam.getPackages())) {
                            //ecw_product_price_step_packaging
                            stepPackagingPrice(prodParam, productPriceDO);
                        }
                        //特性的特殊加价
                        stepSpecialPrice(attrDOMap, prodParam, productPriceDO);

                    } else {
                        // 全包价
                        // TODO 指定包装类型的包装加价
                        if (CollectionUtil.isNotEmpty(prodParam.getPackages())) {
                            List<ProductPricePackagingDO> pricePackagingDOList =
                                    pricePackagingMapper.selectList(new LambdaQueryWrapper<ProductPricePackagingDO>()
                                            .eq(ProductPricePackagingDO::getDeleted, 0)
                                            .eq(ProductPricePackagingDO::getProductPriceId, productPriceDO.getId()));
                            // 筛选匹配包装类型，并比较金额大小，取最大值
                            List<ProductPricePackagingDO> packagingDOList = new ArrayList<>();
                            List<ProductPricePackagingDO> clearancePackagingDOList = new ArrayList<>();
                            for (ProductPricePackagingDO productPricePackagingDO : pricePackagingDOList) {
                                if (StringUtils.isBlank(productPricePackagingDO.getPackagingTypes())) {
                                    continue;
                                }
                                if (packagingTypeVerify(productPricePackagingDO.getPackagingTypes().split(StrUtil.COMMA), prodParam.getPackages())) {
                                    if (productPriceDO.getPriceType() == 1) {
                                        // 全包价
                                        if (productPricePackagingDO.getPriceType() == 1) {
                                            packagingDOList.add(productPricePackagingDO);
                                        }
                                    } else {
                                        // 运费+清关费
                                        if (productPricePackagingDO.getPriceType() == 0) {
                                            // 运费
                                            packagingDOList.add(productPricePackagingDO);
                                        }
                                        if (productPricePackagingDO.getPriceType() == 2) {
                                            // 清关费
                                            clearancePackagingDOList.add(productPricePackagingDO);
                                        }
                                    }
                                }
                            }
                            if (CollectionUtil.isNotEmpty(packagingDOList)) {
                                // 运费、全包价包装价格计算最大值
                                ProductPricePackagingDO packagingDO =
                                        packagingDOList.stream().max(Comparator.comparing(ProductPricePackagingDO::getPackagingPrice)).orElse(null);
                                if (Objects.nonNull(packagingDO)) {
                                    prodParam.setPackagingFreightPrice(packagingDO.getPackagingPrice());
                                    if (!prodParam.getSpecialPriceType()) {
                                        prodParam.setOneFreight(prodParam.getOneFreight().add(packagingDO.getPackagingPrice()));
                                    }
                                } else {
                                    prodParam.setPackagingFreightPrice(BigDecimal.ZERO);
                                }
                            }
                            if (CollectionUtil.isNotEmpty(clearancePackagingDOList)) {
                                // 清关费包装价格计算最大值
                                ProductPricePackagingDO clearancePackagingDO =
                                        clearancePackagingDOList.stream().max(Comparator.comparing(ProductPricePackagingDO::getPackagingPrice)).orElse(null);
                                if (Objects.nonNull(clearancePackagingDO)) {
                                    prodParam.setPackagingClearanceFeePrice(clearancePackagingDO.getPackagingPrice());
                                    if (!prodParam.getSpecialPriceType()) {
                                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(clearancePackagingDO.getPackagingPrice()));
                                    }
                                } else {
                                    prodParam.setPackagingClearanceFeePrice(BigDecimal.ZERO);
                                }
                            }
                            if (StringUtils.isNotBlank(no)) {
                                if (no.startsWith("20")) {
                                    log.info("===== 报价单【{}】 线路包装运费价格：{} ===== 线路包装清关费价格：{} =====", no, JSONObject.toJSONString(packagingDOList), JSONObject.toJSONString(clearancePackagingDOList));
                                } else {
                                    log.info("===== 订单【{}】 线路包装运费价格：{} ===== 线路包装清关费价格：{} =====", no, JSONObject.toJSONString(packagingDOList), JSONObject.toJSONString(clearancePackagingDOList));
                                }
                            } else {
                                log.info("===== 线路包装运费价格：{} ===== 线路包装清关费价格：{} =====", JSONObject.toJSONString(packagingDOList), JSONObject.toJSONString(clearancePackagingDOList));
                            }
                        } else {
                            prodParam.setPackagingFreightPrice(BigDecimal.ZERO);
                            prodParam.setPackagingClearanceFeePrice(BigDecimal.ZERO);
                        }
                        // 特性的特殊加价
                        List<ProductPriceSpecialDO> specialDOList = specialService.list(new LambdaQueryWrapper<ProductPriceSpecialDO>()
                                .eq(ProductPriceSpecialDO::getDeleted, 0)
                                .eq(ProductPriceSpecialDO::getProductPriceId, productPriceDO.getId()));
                        if (StringUtils.isNotBlank(no)) {
                            if (no.startsWith("20")) {
                                log.info("===== 报价单【{}】 线路特殊价格：{} =====", no, JSONObject.toJSONString(specialDOList));
                            } else {
                                log.info("===== 订单【{}】 线路特殊价格：{} =====", no, JSONObject.toJSONString(specialDOList));
                            }
                        } else {
                            log.info("===== 线路特殊价格：{} =====", JSONObject.toJSONString(specialDOList));
                        }
                        if (CollectionUtil.isNotEmpty(specialDOList)) {
                            Map<String, ProductPriceSpecialDO> specialDOMap = specialDOList.stream()
                                    .collect(Collectors.groupingBy(ProductPriceSpecialDO::getSpecialDictType,
                                            Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));

                            // 计算有牌价格
                            if (Objects.nonNull(prodParam.getBrandType()) && prodParam.getBrandType() != 0) {
                                this.addBrandPrice(prodParam, specialDOMap);
                            } else {
                                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
                            }
                            if (Objects.nonNull(attrDOMap)) {
                                // 计算带电价格
                                this.addElectrifiedPrice(attrDOMap, prodParam, specialDOMap);
                                // 计算液体价格
                                this.addLiquidPrice(attrDOMap, prodParam, specialDOMap);
                            }
                        } else {
                            prodParam.setPackagingFreightPrice(BigDecimal.ZERO);
                            prodParam.setPackagingClearanceFeePrice(BigDecimal.ZERO);
                            prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                            prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
                            prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
                            prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
                            prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
                            prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
                        }
                    }
                    //  佣金价格计算
                    if (!prodParam.getSpecialCommission() && Objects.nonNull(customerId)) {
                        //  未设置特价佣金时，每次都获取初始化默认佣金
                        this.getCustomerCommission(customsType, customerId, lineId, prodParam);
                    }
                    if (Objects.nonNull(prodParam.getCommissionType()) && prodParam.getCommissionType() > 0) {
                        this.addCommissionAmount(prodParam);
                    }
                } else {
                    // 当订单运费阶梯原始价格未匹配到，则重置所有特殊加价为0
                    prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                    prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
                    prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
                    prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
                    prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
                    prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
                    prodParam.setCommissionType(0);
                    prodParam.setPlainCommission(BigDecimal.ZERO);
                    prodParam.setHiddenCommission(BigDecimal.ZERO);
                }

            } else {
                zTest3 z = new zTest3();
                z.setCreatedate(new Date());
                String s = "产品线路价格为空";
                z.setTestname(s);
                zTestMapper.insert(z);

                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(transportPrice);
                    prodParam.setOneClearanceFee(clearancePrice);
                }
                prodParam.setOrgClearanceFeePrice(clearancePrice);
                prodParam.setClearanceFee(clearancePrice);
                prodParam.setDiscountClearanceFeePrice(BigDecimal.ZERO);
                prodParam.setDiscountClearanceFeeTotalPrice(BigDecimal.ZERO);
                prodParam.setClearanceFeeCurrency(0); //默认美元
                prodParam.setClearanceFeeVolume(0L);
                prodParam.setOrgFreightPrice(transportPrice);
                prodParam.setDiscountFreightPrice(BigDecimal.ZERO);
                prodParam.setDiscountFreightTotalPrice(BigDecimal.ZERO);
                prodParam.setFreight(transportPrice);
                prodParam.setFreightCurrency(0);
                prodParam.setFreightVolume(0L);
                prodParam.setIsPayAdvance(null);
                prodParam.setCharging(0);
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
                prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
                prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
                prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
                prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
                prodParam.setCommissionType(0);
                prodParam.setPlainCommission(BigDecimal.ZERO);
                prodParam.setHiddenCommission(BigDecimal.ZERO);
            }
        } else {
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            String s = "不符合定价条件";
            z.setTestname(s);
            zTestMapper.insert(z);
        }
    }


    private void stepSpecialPrice(Map<String, List<ProductAttrDO>> attrDOMap, ProdConditionParam prodParam, ProductPriceDO productPriceDO) {
        List<ProductPriceStepSpecialDO> stepSpecialDOList = null;
        if (productPriceDO.getPriceType() == 1) {
            stepSpecialDOList = priceStepSpecialMapper.selectList(new LambdaQueryWrapper<ProductPriceStepSpecialDO>()
                    .eq(ProductPriceStepSpecialDO::getProductPriceStepId, productPriceDO.getAllStepPriceId()));
        } else {
            stepSpecialDOList = priceStepSpecialMapper.selectList(new LambdaQueryWrapper<ProductPriceStepSpecialDO>()
                    .eq(ProductPriceStepSpecialDO::getProductPriceStepId, productPriceDO.getTransportStepPriceId()));
            if (Objects.nonNull(productPriceDO.getClearanceStepPriceId())) {
                List<ProductPriceStepSpecialDO> clearanceStepSpecialDOList = priceStepSpecialMapper.selectList(new LambdaQueryWrapper<ProductPriceStepSpecialDO>()
                        .eq(ProductPriceStepSpecialDO::getProductPriceStepId, productPriceDO.getClearanceStepPriceId()));
                if (CollectionUtil.isNotEmpty(stepSpecialDOList)) {
                    if (CollectionUtil.isNotEmpty(clearanceStepSpecialDOList)) {
                        for (ProductPriceStepSpecialDO clearanceStepSpecialDO : clearanceStepSpecialDOList) {
                            for (ProductPriceStepSpecialDO stepSpecialDO : stepSpecialDOList) {
                                if (StringUtils.equals(stepSpecialDO.getSpecialDictType(), clearanceStepSpecialDO.getSpecialDictType())) {
                                    stepSpecialDO.setClearancePrice(clearanceStepSpecialDO.getClearancePrice());
                                    stepSpecialDO.setClearancePriceUnit(clearanceStepSpecialDO.getClearancePriceUnit());
                                    stepSpecialDO.setClearanceVolumeUnit(clearanceStepSpecialDO.getClearanceVolumeUnit());
                                }
                            }
                        }
                    }
                } else {
                    stepSpecialDOList = clearanceStepSpecialDOList;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(stepSpecialDOList)) {
            Map<String, ProductPriceStepSpecialDO> specialDOMap = stepSpecialDOList.stream()
                    .collect(Collectors.groupingBy(ProductPriceStepSpecialDO::getSpecialDictType,
                            Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));

            // 计算有牌价格
            if (Objects.nonNull(prodParam.getBrandType()) && prodParam.getBrandType() != 0) {
                this.addStepBrandPrice(prodParam, specialDOMap);
            } else {
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            }
            if (Objects.nonNull(attrDOMap)) {
                // 计算带电价格
                this.addStepElectrifiedPrice(attrDOMap, prodParam, specialDOMap);
                // 计算液体价格
                this.addStepLiquidPrice(attrDOMap, prodParam, specialDOMap);
            }
        } else {
            prodParam.setBrandFreightPrice(BigDecimal.ZERO);
            prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
            prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
            prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
            prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
        }
    }


    private void stepPackagingPrice(ProdConditionParam prodParam,
                                    ProductPriceDO productPriceDO) {
        List<ProductPriceStepPackagingDO> stepPackagingDOList;
        List<ProductPriceStepPackagingDO> clearanceStepPackagingDOList = null;

        if (productPriceDO.getPriceType() == 1) {

            stepPackagingDOList = priceStepPackagingMapper.selectList(
                    new LambdaQueryWrapper<ProductPriceStepPackagingDO>()
                    .eq(ProductPriceStepPackagingDO::getProductPriceStepId,
                            productPriceDO.getAllStepPriceId()));
        } else {
            stepPackagingDOList = priceStepPackagingMapper.selectList(
                    new LambdaQueryWrapper<ProductPriceStepPackagingDO>()
                    .eq(ProductPriceStepPackagingDO::getProductPriceStepId,
                            productPriceDO.getTransportStepPriceId()));

            if (Objects.nonNull(productPriceDO.getClearanceStepPriceId())) {
                clearanceStepPackagingDOList =
                        priceStepPackagingMapper.selectList(
                                new LambdaQueryWrapper<ProductPriceStepPackagingDO>()
                        .eq(ProductPriceStepPackagingDO::getProductPriceStepId,
                                productPriceDO.getClearanceStepPriceId()));
            }
        }

        if (CollectionUtil.isNotEmpty(stepPackagingDOList) ||
                CollectionUtil.isNotEmpty(clearanceStepPackagingDOList)) {
            // 包装加价，比较包装类型，并取最大值
            if (stepPackagingDOList != null &&
                    CollectionUtil.isNotEmpty(stepPackagingDOList)) {
                // 比较最大值
                ProductPriceStepPackagingDO stepPackagingDO =
                        stepPackagingDOList.stream().filter(
                                step -> StringUtils.isNotBlank(step.getPackagingTypes()) &&
                                packagingTypeVerify(step.getPackagingTypes().split(StrUtil.COMMA),prodParam.getPackages())).
                                max(Comparator.comparing(ProductPriceStepPackagingDO::getPackagingPrice)).orElse(null);
                if (Objects.nonNull(stepPackagingDO)) {
                    prodParam.setPackagingFreightPrice(
                            stepPackagingDO.getPackagingPrice());

                    if (!prodParam.getSpecialPriceType()) {
                        //运费单价，加包装价格
                        prodParam.setOneFreight(prodParam.getOneFreight().
                                add(stepPackagingDO.getPackagingPrice()));
                    }
                } else {
                    prodParam.setPackagingFreightPrice(BigDecimal.ZERO);
                }
            }

            if (clearanceStepPackagingDOList != null &&
                    CollectionUtil.isNotEmpty(clearanceStepPackagingDOList)) {
                // 比较最大值
                ProductPriceStepPackagingDO clearanceStepPackagingDO =
                        clearanceStepPackagingDOList.stream().
                                filter(step -> StringUtils.isNotBlank(
                                        step.getPackagingTypes())
                                        &&
                                        packagingTypeVerify(step.getPackagingTypes().split(StrUtil.COMMA), prodParam.getPackages())).
                                max(Comparator.comparing(ProductPriceStepPackagingDO::getPackagingPrice)).orElse(null);

                if (Objects.nonNull(clearanceStepPackagingDO)) {
                    prodParam.setPackagingClearanceFeePrice(clearanceStepPackagingDO.getPackagingPrice());
                    if (!prodParam.getSpecialPriceType()) {
                        //清关价单价 加包装价
                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().
                                add(clearanceStepPackagingDO.getPackagingPrice()));
                    }
                } else {
                    prodParam.setPackagingClearanceFeePrice(BigDecimal.ZERO);
                }
            }
        } else {
            prodParam.setPackagingFreightPrice(BigDecimal.ZERO);
            prodParam.setPackagingClearanceFeePrice(BigDecimal.ZERO);
        }
    }

    private boolean packagingTypeVerify(String[] packagingTypes, List<String> packages) {
        return Arrays.stream(packagingTypes).anyMatch(packages::contains);
    }

    /**
     * 处理阶梯价格
     *
     * @param unitDOMap      计量单位键值对
     * @param prodParam      计价参数
     * @param productPriceDO 线路价格主表
     */
    private void stepSelect(Map<Integer, UnitDO> unitDOMap, ProdConditionParam prodParam, ProductPriceDO
            productPriceDO) {
        UnitDO unitDO = unitDOMap.get(Integer.valueOf(prodParam.getFreightVolume().toString()));
        if (unitDO == null) {
            if (!prodParam.getSpecialPriceType()) {
                prodParam.setOneFreight(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
            }
            if (productPriceDO.getPriceType() == 0) {
                productPriceDO.setTransportPrice(BigDecimal.ZERO);
                productPriceDO.setClearancePrice(BigDecimal.ZERO);
            } else {
                productPriceDO.setAllPrice(BigDecimal.ZERO);
            }
        } else {
            BigDecimal num = this.calculationUnitAsValue(prodParam, productPriceDO, unitDO, 2);
            BasePrice priceStep = productPriceStepService.getProductPriceStep(productPriceDO.getId(), num, productPriceDO.getPriceType());
            if (Objects.nonNull(priceStep)) {
                log.info("===== 线路无牌无液无电阶梯价：{} =====", JSONObject.toJSONString(priceStep));
                if (productPriceDO.getPriceType() == 0) {
                    productPriceDO.setTransportPrice(priceStep.getTransportPrice());
                    productPriceDO.setClearancePrice(priceStep.getClearancePrice());
                    productPriceDO.setTransportStepPriceId(priceStep.getTransportStepPriceId());
                } else {
                    productPriceDO.setAllPrice(priceStep.getAllPrice());
                    productPriceDO.setAllStepPriceId(priceStep.getAllStepPriceId());
                }
            } else {
                log.info("===== 线路无牌无液无电阶梯价不存在【{} {}】该档次 =====", num, unitDO.getFuhao());
                // 未报价 不给是否预付的状态
                productPriceDO.setNeedPay(null);
                if (productPriceDO.getPriceType() == 0) {
                    productPriceDO.setTransportPrice(BigDecimal.ZERO);
                    productPriceDO.setClearancePrice(BigDecimal.ZERO);
                } else {
                    productPriceDO.setAllPrice(BigDecimal.ZERO);
                }
            }
        }
    }

    /**
     * @param prodParam      品名参数
     * @param productPriceDO 价格参数
     * @param unitDO         计量单位
     * @param type           类型：0 空运清关费入仓值为准 1 空运运费收费值为准 2 海运计算
     * @return 计量值
     */
    private BigDecimal calculationUnitAsValue(ProdConditionParam prodParam,
                                              ProductPriceDO productPriceDO,
                                              UnitDO unitDO,
                                              Integer type) {

        BigDecimal num;
        if (StringUtils.equals("m³", unitDO.getFuhao())) {
            // 体积
            if ((null == prodParam.getVolume() || prodParam.getVolume().compareTo(BigDecimal.ZERO) <= 0) && (Objects.isNull(prodParam.getChargeVolume()) || prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) <= 0)) {
                num = Objects.isNull(productPriceDO.getMinWeight()) || productPriceDO.getMinWeight().compareTo(BigDecimal.ZERO) == 0 ? new BigDecimal("0.01") : productPriceDO.getMinWeight();
            } else {
                num = Objects.isNull(prodParam.getVolume()) ? BigDecimal.ZERO : prodParam.getVolume();
                if (type == 1 && Objects.nonNull(prodParam.getChargeTotalVolume()) && prodParam.getChargeTotalVolume().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeTotalVolume();
                }
                if (type == 2 && Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeVolume();
                }
                if (type == 2) {
                    num = (num.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
                            num : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
                }
            }

        } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
            if ((null == prodParam.getWeight() || prodParam.getWeight().compareTo(BigDecimal.ZERO) <= 0) && (Objects.isNull(prodParam.getChargeWeight()) || prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) <= 0)) {
                num = Objects.isNull(productPriceDO.getMinWeight()) || productPriceDO.getMinWeight().compareTo(BigDecimal.ZERO) == 0 ? new BigDecimal("0.01") : productPriceDO.getMinWeight();
            } else {
                num = Objects.isNull(prodParam.getWeight()) ? BigDecimal.ZERO : prodParam.getWeight();
                if (type == 1 && Objects.nonNull(prodParam.getChargeTotalWeight()) && prodParam.getChargeTotalWeight().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeTotalWeight();
                }
                if (type == 2 && Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeWeight();
                }
                if (type == 2) {
                    num = (num.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
                            num : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
                }
            }
        } else if (StringUtils.equals("a", unitDO.getFuhao())) {
            if ((null == prodParam.getQuantity() || prodParam.getQuantity() <= 0) && (Objects.isNull(prodParam.getChargeQuantity()) || prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) <= 0)) {
                num = Objects.isNull(productPriceDO.getMinWeight()) || productPriceDO.getMinWeight().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ONE : productPriceDO.getMinWeight();
            } else {
                // 数量
                // 数量计算
                num = Objects.nonNull(prodParam.getQuantity()) ? new BigDecimal(prodParam.getQuantity()) : BigDecimal.ZERO;
                if (type == 1 && Objects.nonNull(prodParam.getChargeTotalQuantity()) && prodParam.getChargeTotalQuantity().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeTotalQuantity();
                }
                if (type == 2 && Objects.nonNull(prodParam.getChargeQuantity()) && prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) > 0) {
                    num = prodParam.getChargeQuantity();
                }
                if (type == 2) {
                    num = num.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? num : prodParam.getMinMeteringQuantity();
                }
            }
        } else {
            throw exception(UNIT_NOT_LEGAL);
        }
        return num;
    }


    /**
     * 处理阶梯价格
     *
     * @param unitDOMap      计量单位键值对
     * @param prodParam      计价参数
     * @param productPriceDO 线路价格主表
     * @param lineId         线路ID
     * @param channelId      渠道ID
     */
    private void airStepSelect(Map<Integer, UnitDO> unitDOMap,
                               ProdConditionParam prodParam,
                               ProductPriceDO productPriceDO,
                               Long lineId, Long channelId) {

        if (prodParam.getUnitDO() == null) {
            //运费计量单位信息
            if (productPriceDO.getPriceType() == 0) {
                //单价模式，0-运费+清关费，1-全包价
                productPriceDO.setTransportPrice(BigDecimal.ZERO);
            } else {
                productPriceDO.setAllPrice(BigDecimal.ZERO);
            }

            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("2运费计量单位 为空");
            zTestMapper.insert(z);
            return;
        }

        //计量单位符号
        BigDecimal num = this.calculationUnitAsValue(prodParam,
                productPriceDO,
                prodParam.getUnitDO(),
                1);

        // 添加的逻辑
        if (productPriceDO.getPriceType() == null) {
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("3价格类型 为空");
            zTestMapper.insert(z);
            return;
        }

        //全包价
        if (productPriceDO.getPriceType() == 1) {
            BasePrice allBasePrice =
                    productPriceStepService.getProductPriceStep(
                            productPriceDO.getId(), num, 1);
            if (Objects.nonNull(allBasePrice)) {
                log.info("===== 线路无牌无液无电全包阶梯价：{} =====",
                        JSONObject.toJSONString(allBasePrice));
                productPriceDO.setAllPrice(allBasePrice.getAllPrice());
                productPriceDO.setAllStepPriceId(allBasePrice.getAllStepPriceId());
            } else {
                log.info("===== 线路无牌无液无电全包阶梯价不存在【{} {}】该档次 =====",
                        num, prodParam.getUnitDO().getFuhao());
                productPriceDO.setNeedPay(null);
                productPriceDO.setAllPrice(BigDecimal.ZERO);
            }
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("4价格类型 为全包价，无需计算清关费");
            zTestMapper.insert(z);
            return;
        }

        //0-运费+清关费 才走下面的逻辑
        //单价模式，0-运费+清关费，1-全包价
        BasePrice transportBasePrice =
                productPriceStepService.getProductPriceStep(
                        productPriceDO.getId(), num, 0);
        if (Objects.nonNull(transportBasePrice)) {
            log.info("线路无牌无液无电运费阶梯价：{} =====",
                    JSONObject.toJSONString(transportBasePrice));
            productPriceDO.setTransportPrice(
                    transportBasePrice.getTransportPrice());
            productPriceDO.setTransportStepPriceId(
                    transportBasePrice.getTransportStepPriceId());
        } else {
            productPriceDO.setNeedPay(null);
            productPriceDO.setTransportPrice(BigDecimal.ZERO);

            String s = "5线路无牌无液无电运费阶梯价不存在【" + num +
                    prodParam.getUnitDO().getFuhao() + "】该档次";
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname(s);
            zTestMapper.insert(z);
        }

        if (prodParam.getClearanceUnitDO() == null) {
            //清关费单位为空，设置清关费为0
            productPriceDO.setClearancePrice(BigDecimal.ZERO);
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("6清关费单位为空");
            zTestMapper.insert(z);
            return;
        }

        // 清关费以实际入仓值来匹配
        // 2024-1-17 空运清关费改为和空运运费一样使用订单总收费值来匹配
        //BigDecimal clearanceNum = this.calculationUnitAsValue(prodParam, productPriceDO, prodParam.getClearanceUnitDO(), 0);
        //清关价计量单位
        BigDecimal clearanceNum = num;
        //获取阶梯价
        BasePrice clearanceBasePrice =
                productPriceStepService.getProductPriceStep(
                        productPriceDO.getId(),
                        clearanceNum, 2);
        clearanceBasePrice.setClearancePriceUnit(
                productPriceDO.getClearancePriceUnit());
        clearanceBasePrice.setClearanceVolumeUnit(
                productPriceDO.getClearanceVolumeUnit());

        log.info("线路无牌无液无电清关费阶梯价：{}",
                JSONObject.toJSONString(clearanceBasePrice));

        int airClearanceSource = 3;
        BasePrice clearancePrice;
        //clearanceBasePrice 阶梯价
        //线路存在，产品路线价格存在清关价
        if (clearanceBasePrice.getProductPriceEmptyType() == 0 &&
                Objects.nonNull(lineId) && lineId > 0
                && Objects.nonNull(productPriceDO.getClearancePrice())
                && productPriceDO.getClearancePrice().compareTo(BigDecimal.ZERO) > 0) {


            if (CollectionUtil.isNotEmpty(prodParam.getPackages())) {
                zTest3 z = new zTest3();
                z.setCreatedate(new Date());
                z.setTestname("7产品包装类型list为：" +
                        prodParam.getPackages().toString());
                zTestMapper.insert(z);
            } else {
                zTest3 z = new zTest3();
                z.setCreatedate(new Date());
                String s = "8产品包装类型List为空";
                if (prodParam.getPackaging() == null) {
                    s += ",包装类型也为空";
                } else {
                    s += ",包装类型" + prodParam.getPackaging();
                }

                z.setTestname(s);
                zTestMapper.insert(z);
            }
            //TODO 获取线路设置的空运清关费
            //PACKAGING_TYPE 包装要求 数据字典中的数据
            // 1箱，2编织袋，3木架，4木箱，5托，6航空箱，7桶，8压缩包，9铁架
            //线路渠道清关费包装类型表 ecw_line_channel_packaging
            //ecw_line_channel_price_step_clearance 清关费阶梯价
            List<LineChannelPackagingDO> lineChannelPackagingDOList =
                    lineChannelPackagingService.queryAirClearancePrice(
                            CollectionUtil.isNotEmpty(prodParam.getPackages()) ?
                                    prodParam.getPackages() :
                                    Collections.singletonList(prodParam.getPackaging()),
                            channelId, lineId);

            if (CollectionUtil.isNotEmpty(lineChannelPackagingDOList)) {
                //如果线路渠道的包装类型为空，就取阶梯价的清关价
                BasePrice lineClearanceBasePrice = null;

                for (LineChannelPackagingDO lineChannelPackagingDO :
                        lineChannelPackagingDOList) {

                    //计量单位
                    UnitDO lineClearanceUnitDO =
                            unitDOMap.get(Integer.valueOf(
                                    lineChannelPackagingDO.getClearanceVolumeUnit().toString()));
                    if (lineClearanceUnitDO == null) {
                        continue;
                    }

                    if (Objects.isNull(lineClearanceBasePrice)) {
                        lineClearanceBasePrice = new BasePrice();
                        lineClearanceBasePrice.setClearancePrice(BigDecimal.ZERO);
                        lineClearanceBasePrice.setProductClearanceAsExchangePrice(BigDecimal.ZERO);
                    }
                    lineClearanceBasePrice.setClearancePriceUnit(
                            lineChannelPackagingDO.getClearancePriceUnit());
                    lineClearanceBasePrice.setClearanceVolumeUnit(
                            lineChannelPackagingDO.getClearanceVolumeUnit());
                    // TODO 清关费也以收费值进行匹配
                    BigDecimal lineClearanceNum =
                            this.calculationUnitAsValue(prodParam,
                                    productPriceDO, lineClearanceUnitDO, 1);

                    if (lineClearanceNum != null) {
                        zTest3 z = new zTest3();
                        z.setCreatedate(new Date());
                        z.setTestname("99lineClearanceNum=" + lineClearanceNum);
                        zTestMapper.insert(z);
                    }
                    //ecw_line_channel_price_step_clearance
                    //线路渠道清关费阶梯价格
                    List<LineChannelPriceStepClearanceDO>
                            packagingStepPriceDOList =
                            lineChannelPackagingDO.getLineChannelPriceStepClearanceDOList();
                    //币种
                    ExchangeRateRespDTO productExchangeRate =
                            currencyApi.getCurrencyRate(
                                    lineClearanceBasePrice.getClearancePriceUnit(),
                                    1L);

                    //线路渠道清关费阶梯价格
                    if (CollectionUtil.isNotEmpty(packagingStepPriceDOList)) {

                        LineChannelPriceStepClearanceDO priceStepDO =
                                packagingStepPriceDOList.stream().
                                        filter(priceStep -> priceStep.getStartNum().
                                                compareTo(lineClearanceNum) <= 0
                                                && priceStep.getEndNum().compareTo(
                                                lineClearanceNum) >= 0).
                                        findFirst().orElse(null);

                        if (Objects.nonNull(priceStepDO)) {

                            BigDecimal productClearancePrice =
                                    productExchangeRate.getCurrencyRate().
                                            multiply(priceStepDO.getClearancePrice()).
                                            setScale(2, BigDecimal.ROUND_HALF_UP);

                            if (lineClearanceBasePrice.getClearancePrice().
                                    compareTo(BigDecimal.ZERO) > 0) {
                                if (productClearancePrice.compareTo(
                                        lineClearanceBasePrice.
                                                getProductClearanceAsExchangePrice()) > 0) {

                                    lineClearanceBasePrice.setClearanceStepPriceId(
                                            priceStepDO.getId());
                                    lineClearanceBasePrice.setAirClearanceSource(4);
                                    lineClearanceBasePrice.setClearancePrice(
                                            priceStepDO.getClearancePrice());
                                    lineClearanceBasePrice.setAirClearanceInfo(
                                            JSONObject.toJSONString(priceStepDO));
                                }
                            } else {
                                BeanUtils.copyProperties(priceStepDO, lineClearanceBasePrice);
                                lineClearanceBasePrice.setClearanceStepPriceId(
                                        priceStepDO.getId());
                                lineClearanceBasePrice.setAirClearanceSource(4);
                                lineClearanceBasePrice.setClearancePrice(
                                        priceStepDO.getClearancePrice());
                                lineClearanceBasePrice.setAirClearanceInfo(
                                        JSONObject.toJSONString(priceStepDO));
                                lineClearanceBasePrice.setProductClearanceAsExchangePrice(productClearancePrice);
                            }
                        }
                    } else {
                        //线路渠道清关费阶梯价格 为空
                        zTest3 z = new zTest3();
                        z.setCreatedate(new Date());
                        z.setTestname("9线路渠道清关费阶梯价格 为空lineId="
                                + lineId + "channelId=" + channelId + "lineClearanceNum=" +
                                lineClearanceNum);
                        zTestMapper.insert(z);
                    }
                }

                if (lineClearanceBasePrice != null) {

                    int result = 0;
                    if (!clearanceBasePrice.getClearancePriceUnit().
                            equals(lineClearanceBasePrice.getClearancePriceUnit())) {

                        ExchangeRateRespDTO productExchangeRate =
                                currencyApi.getCurrencyRate(
                                        clearanceBasePrice.getClearancePriceUnit(),
                                        1L);
                        BigDecimal productClearancePrice =
                                productExchangeRate.getCurrencyRate().
                                        multiply(clearanceBasePrice.getClearancePrice()).
                                        setScale(2,
                                                BigDecimal.ROUND_HALF_UP);
                        BigDecimal lineClearancePrice =
                                lineClearanceBasePrice.getProductClearanceAsExchangePrice();
                        result = productClearancePrice.compareTo(lineClearancePrice);
                    } else {
                        result = clearanceBasePrice.getClearancePrice().
                                compareTo(lineClearanceBasePrice.getClearancePrice());
                    }
                    //清关费来源
                    airClearanceSource = result >= 0 ? 3 : 4;

                    clearancePrice = result >= 0 ? clearanceBasePrice :
                            lineClearanceBasePrice;

                    //clearancePrice=lineClearanceBasePrice;

                } else {
                    clearancePrice = clearanceBasePrice;
                }
            } else {
                zTest3 z = new zTest3();
                z.setCreatedate(new Date());
                z.setTestname("10线路渠道包装类型为空lineId="
                        + lineId + "channelId=" + channelId);
                zTestMapper.insert(z);
                //产品阶梯价的清关价
                clearancePrice = clearanceBasePrice;
            }
            //设置清关价
            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("计算出的清关价为：" +
                    clearancePrice.getClearancePrice());
            zTestMapper.insert(z);

            productPriceDO.setClearancePrice(
                    clearancePrice.getClearancePrice());

            productPriceDO.setClearanceStepPriceId(
                    clearancePrice.getClearanceStepPriceId());
            productPriceDO.setProductPriceEmptyType(
                    clearancePrice.getProductPriceEmptyType());
            productPriceDO.setAirClearanceInfo(
                    clearancePrice.getAirClearanceInfo());      // 清关费详情
            prodParam.setAirClearanceSource(airClearanceSource);// 清关费来源
            prodParam.setAirClearanceInfo(
                    productPriceDO.getAirClearanceInfo());      // 清关费详情

        } else {

            zTest3 z = new zTest3();
            z.setCreatedate(new Date());
            z.setTestname("线路无牌无液无电清关费阶梯价不存在，不符合逻辑条件【"
                    + num +
                    prodParam.getClearanceUnitDO().getFuhao() + "】该档次"
            );

            zTestMapper.insert(z);
            productPriceDO.setClearancePrice(BigDecimal.ZERO);
        }
    }

    private void getCustomerCommission(Integer customsType, Long customerId, Long lineId, ProdConditionParam
            prodParam) {
        CustomerCommissionQueryDto queryDto = new CustomerCommissionQueryDto();
        queryDto.setCustomerId(customerId);
        queryDto.setLineId(lineId);
        queryDto.setProductId(prodParam.getProdId());
        queryDto.setCustomsType(customsType);
        queryDto.setApproval(prodParam.getBrandType());
        queryDto.setOrgFreight(prodParam.getOrgFreightPrice());
        queryDto.setFreightCurrencyId(prodParam.getFreightCurrency());
        queryDto.setFreightVolumeId(prodParam.getFreightVolume());
        CustomerCommissionDto customerCommissionDto = customerCommissionInfoService.customerCommission(queryDto);
        if (Objects.nonNull(customerCommissionDto)) {
            prodParam.setCommissionType(customerCommissionDto.getCommissionType());
            prodParam.setPlainCommission(customerCommissionDto.getPlainCommission());
            prodParam.setHiddenCommission(customerCommissionDto.getHiddenCommission());
        } else {
            prodParam.setCommissionType(0);
            prodParam.setPlainCommission(BigDecimal.ZERO);
            prodParam.setHiddenCommission(BigDecimal.ZERO);
        }
    }

    private void addCommissionAmount(ProdConditionParam prodParam) {
        if (1 == prodParam.getCommissionType() || 3 == prodParam.getCommissionType()) {
            // 只有明佣加入到计价的成交单价中，暗佣是包含在原价中的
            if (!prodParam.getSpecialPriceType()) {
                if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(prodParam.getPlainCommission()));
                }
            }
        }
    }

    private void addStepBrandPrice(ProdConditionParam
                                           prodParam, Map<String, ProductPriceStepSpecialDO> specialDOMap) {
        if (1 == prodParam.getBrandType()) {
            // 有牌价格计算
            ProductPriceStepSpecialDO specialDO = specialDOMap.get("licensed_price");
            if (null != specialDO) {
                BigDecimal brandFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setBrandFreightPrice(brandFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(brandFreightPrice));
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal brandClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setBrandClearanceFeePrice(brandClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice()));
                    }
                }
            } else {
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            }
        }
        if (2 == prodParam.getBrandType()) {
            // 中性价格计算
            ProductPriceStepSpecialDO specialDO = specialDOMap.get("neutral_brand_prices");
            if (null != specialDO) {
                BigDecimal brandFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setBrandFreightPrice(brandFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(brandFreightPrice));
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal brandClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setBrandClearanceFeePrice(brandClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(brandClearancePrice));
                    }
                }
            } else {
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            }
        }


    }

    private void addStepElectrifiedPrice(Map<String, List<ProductAttrDO>> attrDOMap, ProdConditionParam
            prodParam, Map<String, ProductPriceStepSpecialDO> specialDOMap) {
        if (null != attrDOMap.get("L")) {
            ProductPriceStepSpecialDO specialDO = specialDOMap.get("live_price");
            if (specialDO != null) {
                BigDecimal liveFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setElectrifiedFreightPrice(liveFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                        prodParam.setOneFreight(prodParam.getOneFreight().add(liveFreightPrice));
                    }
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal liveClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setElectrifiedClearanceFeePrice(liveClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                            prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(liveClearancePrice));
                        }
                    }
                }
            } else {
                prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
                prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
            }
        } else {
            prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
            prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
        }
    }


    private void addStepLiquidPrice(Map<String, List<ProductAttrDO>> attrDOMap, ProdConditionParam
            prodParam, Map<String, ProductPriceStepSpecialDO> specialDOMap) {
        if (null != attrDOMap.get("D")) {
            ProductPriceStepSpecialDO specialDO = specialDOMap.get("liquid_price");
            if (specialDO != null) {
                BigDecimal liquidFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setLiquidFreightPrice(liquidFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                        prodParam.setOneFreight(prodParam.getOneFreight().add(liquidFreightPrice));
                    }
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal liquidClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setLiquidClearanceFeePrice(liquidClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                            prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(liquidClearancePrice));
                        }
                    }
                }
            } else {
                prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
                prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
            }
        } else {
            prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
            prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
        }
    }

    private void addBrandPrice(ProdConditionParam prodParam, Map<String, ProductPriceSpecialDO> specialDOMap) {
        if (1 == prodParam.getBrandType()) {
            // 有牌价格计算
            ProductPriceSpecialDO specialDO = specialDOMap.get("licensed_price");
            if (null != specialDO) {
                BigDecimal brandFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setBrandFreightPrice(brandFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(brandFreightPrice));
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal brandClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setBrandClearanceFeePrice(brandClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice()));
                    }
                }
            } else {
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            }
        }
        if (2 == prodParam.getBrandType()) {
            // 中性价格计算
            ProductPriceSpecialDO specialDO = specialDOMap.get("neutral_brand_prices");
            if (null != specialDO) {
                BigDecimal brandFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setBrandFreightPrice(brandFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    prodParam.setOneFreight(prodParam.getOneFreight().add(brandFreightPrice));
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal brandClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setBrandClearanceFeePrice(brandClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(brandClearancePrice));
                    }
                }
            } else {
                prodParam.setBrandFreightPrice(BigDecimal.ZERO);
                prodParam.setBrandClearanceFeePrice(BigDecimal.ZERO);
            }
        }


    }

    private void addElectrifiedPrice(Map<String, List<ProductAttrDO>> attrDOMap, ProdConditionParam
            prodParam, Map<String, ProductPriceSpecialDO> specialDOMap) {
        if (null != attrDOMap.get("L")) {
            ProductPriceSpecialDO specialDO = specialDOMap.get("live_price");
            if (specialDO != null) {
                BigDecimal liveFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setElectrifiedFreightPrice(liveFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                        prodParam.setOneFreight(prodParam.getOneFreight().add(liveFreightPrice));
                    }
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal liveClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setElectrifiedClearanceFeePrice(liveClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                            prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(liveClearancePrice));
                        }
                    }
                }
            } else {
                prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
                prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
            }
        } else {
            prodParam.setElectrifiedFreightPrice(BigDecimal.ZERO);
            prodParam.setElectrifiedClearanceFeePrice(BigDecimal.ZERO);
        }
    }


    private void addLiquidPrice(Map<String, List<ProductAttrDO>> attrDOMap, ProdConditionParam
            prodParam, Map<String, ProductPriceSpecialDO> specialDOMap) {
        if (null != attrDOMap.get("D")) {
            ProductPriceSpecialDO specialDO = specialDOMap.get("liquid_price");
            if (specialDO != null) {
                BigDecimal liquidFreightPrice = prodParam.getCharging() == 0 ?
                        (Objects.isNull(specialDO.getTransportPrice()) ? BigDecimal.ZERO : specialDO.getTransportPrice()) :
                        (Objects.isNull(specialDO.getAllPrice()) ? BigDecimal.ZERO : specialDO.getAllPrice());
                prodParam.setLiquidFreightPrice(liquidFreightPrice);
                if (!prodParam.getSpecialPriceType()) {
                    if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                        prodParam.setOneFreight(prodParam.getOneFreight().add(liquidFreightPrice));
                    }
                }
                if (prodParam.getCharging() == 0) {
                    BigDecimal liquidClearancePrice = Objects.isNull(specialDO.getClearancePrice()) ? BigDecimal.ZERO : specialDO.getClearancePrice();
                    prodParam.setLiquidClearanceFeePrice(liquidClearancePrice);
                    if (!prodParam.getSpecialPriceType()) {
                        if (prodParam.getOneFreight().compareTo(BigDecimal.ZERO) > 0) {
                            prodParam.setOneClearanceFee(prodParam.getOneClearanceFee().add(liquidClearancePrice));
                        }
                    }
                }
            } else {
                prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
                prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
            }
        } else {
            prodParam.setLiquidFreightPrice(BigDecimal.ZERO);
            prodParam.setLiquidClearanceFeePrice(BigDecimal.ZERO);
        }
    }

    private void setReceivableTransportPrice(Map<Integer, UnitDO> unitDOMap, ProdConditionParam prodParam) {
        // 计算订单应收运费
        if (Objects.isNull(prodParam.getOneFreight()) || prodParam.getOneFreight().compareTo(BigDecimal.ZERO) == 0) {
            prodParam.setOneFreight(BigDecimal.ZERO);
            prodParam.setReceivableOneSeaFreight(BigDecimal.ZERO);
            prodParam.setReceivableSeaFreight(prodParam.getReceivableOneSeaFreight());
        } else {
            UnitDO unitDO = unitDOMap.get(Integer.valueOf(prodParam.getFreightVolume().toString()));
            if (unitDO == null) {
                prodParam.setFreight(BigDecimal.ZERO);
            } else {
                if (StringUtils.equals("m³", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeVolume() || prodParam.getOrgChargeVolume().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableSeaFreight(BigDecimal.ZERO);
                    } else {
                        // 体积计算
                        BigDecimal volume = prodParam.getOrgChargeVolume();
                        if (volume.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单体积大于0时，才比较最小收费计量
                            volume = (volume.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
                                    volume : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
                        }
                        prodParam.setReceivableSeaFreight(prodParam.getReceivableOneSeaFreight()
                                .multiply(volume).setScale(0, RoundingMode.HALF_UP));
                    }
                } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeWeight() || prodParam.getOrgChargeWeight().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableSeaFreight(BigDecimal.ZERO);
                    } else {
                        BigDecimal weight = prodParam.getOrgChargeWeight();
                        if (weight.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单重量大于0时，才比较最小收费计量
                            weight = (weight.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
                                    weight : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
                        }
                        prodParam.setReceivableSeaFreight(prodParam.getReceivableOneSeaFreight()
                                .multiply(weight).setScale(0, RoundingMode.HALF_UP));
                    }
                } else if (StringUtils.equals("a", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeQuantity() || prodParam.getOrgChargeQuantity().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableSeaFreight(BigDecimal.ZERO);
                    } else {
                        // 数量计算
                        BigDecimal quantity = prodParam.getOrgChargeQuantity();
                        if (quantity.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单数量大于0时，才比较最小收费计量
                            quantity = quantity.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? quantity : prodParam.getMinMeteringQuantity();
                        }
                        prodParam.setReceivableSeaFreight(quantity.multiply(prodParam.getReceivableOneSeaFreight()).setScale(0, RoundingMode.HALF_UP));
                    }
                } else {
                    throw exception(UNIT_NOT_LEGAL);
                }
            }
        }

    }


    private void setTransportPrice(Map<Integer, UnitDO> unitDOMap, ProdConditionParam prodParam) {
        if (Objects.isNull(prodParam.getOneFreight()) || prodParam.getOneFreight().compareTo(BigDecimal.ZERO) == 0) {
            prodParam.setOneFreight(BigDecimal.ZERO);
            prodParam.setFreight(prodParam.getOneFreight());
            BigDecimal notDiscountFreight = prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice());
            prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
        } else {
            if (Objects.isNull(prodParam.getDiscountFreightPrice())) {
                prodParam.setDiscountFreightPrice(BigDecimal.ZERO);
            }
            // TODO 重泡货判断
            UnitDO unitDO = unitDOMap.get(Integer.valueOf(prodParam.getFreightVolume().toString()));
            if (unitDO == null) {
                prodParam.setFreight(BigDecimal.ZERO);
            } else {
                if (StringUtils.equals("m³", unitDO.getFuhao())) {
                    if (null == prodParam.getVolume() || prodParam.getVolume().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setFreight(BigDecimal.ZERO);
                        BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(prodParam.getMinMeteringVolume()).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
                    } else {
//                        if (Objects.isNull(prodParam.getItemType()) || prodParam.getItemType() == 0 || prodParam.getItemType() == 1) {
                        // 体积计算
                        BigDecimal volume = prodParam.getVolume();
                        if (Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0) {
                            volume = prodParam.getChargeVolume();
                        }
                        if (volume.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单体积大于0时，才比较最小收费计量
                            volume = (volume.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
                                    volume : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
                        }
                        prodParam.setFreight(prodParam.getOneFreight()
                                .multiply(volume).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(volume).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
//                        } else {
//                            // 体积计算
//                            BigDecimal volume = Objects.isNull(prodParam.getWVolume()) ?
//                                    prodParam.getVolume() : prodParam.getWVolume();
//                            if (Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0) {
//                                volume = prodParam.getChargeVolume();
//                            }
//                            volume = (volume.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
//                                    volume : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
//                            prodParam.setFreight(prodParam.getOneFreight().multiply(volume).setScale(0, RoundingMode.HALF_UP));
//                            BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(volume).setScale(0, RoundingMode.HALF_UP);
//                            prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
//                        }
                    }
                } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
                    if (null == prodParam.getWeight() || prodParam.getWeight().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setFreight(BigDecimal.ZERO);
                        BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(prodParam.getMinMeteringWeight()).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
                    } else {
//                        if (Objects.isNull(prodParam.getItemType()) || prodParam.getItemType() == 0 || prodParam.getItemType() == 1) {
                        BigDecimal weight = prodParam.getWeight();
                        if (Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0) {
                            weight = prodParam.getChargeWeight();
                        }
                        if (weight.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单重量大于0时，才比较最小收费计量
                            weight = (weight.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
                                    weight : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
                        }
                        prodParam.setFreight(prodParam.getOneFreight()
                                .multiply(weight).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(weight).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
//                        } else {
//                            // 重量计算
//                            BigDecimal weight = Objects.isNull(prodParam.getVWeight()) ? prodParam.getWeight() : prodParam.getVWeight();
//                            if (Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0) {
//                                weight = prodParam.getChargeVolume();
//                            }
//                            weight = (weight.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
//                                    weight : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
//                            prodParam.setFreight(prodParam.getOneFreight().multiply(weight).setScale(0, RoundingMode.HALF_UP));
//                            BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(weight).setScale(0, RoundingMode.HALF_UP);
//                            prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
//                        }
                    }
                } else if (StringUtils.equals("a", unitDO.getFuhao())) {
                    if ((null == prodParam.getQuantity() || prodParam.getQuantity() <= 0) && (Objects.isNull(prodParam.getChargeQuantity()) || prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) <= 0)) {
                        prodParam.setFreight(BigDecimal.ZERO);
                    } else {
                        // 数量计算
                        BigDecimal quantity = new BigDecimal(prodParam.getQuantity());
                        if (Objects.nonNull(prodParam.getChargeQuantity()) && prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) > 0) {
                            quantity = prodParam.getChargeQuantity();
                        }
                        if (quantity.compareTo(BigDecimal.ZERO) > 0) {
                            // 当收费或填单数量大于0时，才比较最小收费计量
                            quantity = quantity.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? quantity : prodParam.getMinMeteringQuantity();
                        }
                        prodParam.setFreight(quantity.multiply(prodParam.getOneFreight()).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountFreight = (prodParam.getOneFreight().add(prodParam.getDiscountFreightPrice())).multiply(quantity).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountFreightTotalPrice(notDiscountFreight.subtract(prodParam.getFreight()));
                    }
                } else {
                    throw exception(UNIT_NOT_LEGAL);
                }
            }
        }

    }

    private void setReceivableClearancePrice(Map<Integer, UnitDO> unitDOMap, ProdConditionParam
            prodParam, Integer transportId) {
        if (Objects.isNull(prodParam.getOneClearanceFee()) || prodParam.getOneClearanceFee().compareTo(BigDecimal.ZERO) == 0) {
            prodParam.setOneClearanceFee(BigDecimal.ZERO);
            prodParam.setReceivableOneClearanceFreight(BigDecimal.ZERO);
            prodParam.setReceivableClearanceFreight(prodParam.getReceivableOneClearanceFreight());
        } else {
            UnitDO unitDO = unitDOMap.get(Integer.valueOf(prodParam.getClearanceFeeVolume().toString()));
            if (unitDO == null) {
                prodParam.setClearanceFee(BigDecimal.ZERO);
            } else {
                if (StringUtils.equals("m³", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeVolume() || prodParam.getOrgChargeVolume().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableClearanceFreight(BigDecimal.ZERO);
                    } else {
                        BigDecimal volume = prodParam.getOrgChargeVolume();
                        if (transportId == 1 || transportId == 2) {
                            if (volume.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单体积大于0时，才比较最小收费计量
                                volume = (volume.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
                                        volume : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
                            }
                        } else {
                            // 空运相关取实际体积
                            volume = prodParam.getVolume();
                        }
                        prodParam.setReceivableClearanceFreight(prodParam.getReceivableOneClearanceFreight()
                                .multiply(volume).setScale(0, RoundingMode.HALF_UP));
                    }
                } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeWeight() || prodParam.getOrgChargeWeight().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableClearanceFreight(BigDecimal.ZERO);
                    } else {
                        BigDecimal weight = prodParam.getOrgChargeWeight();
                        if (transportId == 1 || transportId == 2) {
                            if (weight.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单重量大于0时，才比较最小收费计量
                                weight = (weight.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
                                        weight : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
                            }
                        } else {
                            // 空运相关取实际重量
                            weight = prodParam.getWeight();
                        }
                        prodParam.setReceivableClearanceFreight(prodParam.getReceivableOneClearanceFreight()
                                .multiply(weight).setScale(0, RoundingMode.HALF_UP));
                    }
                } else if (StringUtils.equals("a", unitDO.getFuhao())) {
                    if (null == prodParam.getOrgChargeQuantity() || prodParam.getOrgChargeQuantity().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setReceivableClearanceFreight(BigDecimal.ZERO);
                    } else {
                        // 数量计算
                        BigDecimal quantity = prodParam.getOrgChargeQuantity();
                        if (transportId == 1 || transportId == 2) {
                            if (quantity.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单数量大于0时，才比较最小收费计量
                                quantity = quantity.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? quantity : prodParam.getMinMeteringQuantity();
                            }
                        } else {
                            // 空运相关取实际数量
                            quantity = Objects.isNull(prodParam.getQuantity()) ? BigDecimal.ZERO : new BigDecimal(prodParam.getQuantity());
                        }
                        prodParam.setReceivableClearanceFreight(quantity.multiply(prodParam.getReceivableOneClearanceFreight()).setScale(0, RoundingMode.HALF_UP));
                    }
                } else {
                    throw exception(UNIT_NOT_LEGAL);
                }
            }
        }
    }

    private void setClearancePrice(Map<Integer, UnitDO> unitDOMap, ProdConditionParam prodParam, Integer
            transportId) {
        if (Objects.isNull(prodParam.getOneClearanceFee()) || prodParam.getOneClearanceFee().compareTo(BigDecimal.ZERO) == 0) {
            prodParam.setOneClearanceFee(BigDecimal.ZERO);
            prodParam.setClearanceFee(prodParam.getOneClearanceFee());
            BigDecimal notDiscountClearanceFeePrice = prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice());
            prodParam.setDiscountClearanceFeePrice(notDiscountClearanceFeePrice.subtract(prodParam.getClearanceFee()));
        } else {
            if (Objects.isNull(prodParam.getDiscountClearanceFeePrice())) {
                prodParam.setDiscountClearanceFeePrice(BigDecimal.ZERO);
            }
            UnitDO unitDO = unitDOMap.get(Integer.valueOf(prodParam.getClearanceFeeVolume().toString()));
            if (unitDO == null) {
                prodParam.setClearanceFee(BigDecimal.ZERO);
            } else {
                if (StringUtils.equals("m³", unitDO.getFuhao())) {
                    if (null == prodParam.getVolume() || prodParam.getVolume().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setClearanceFee(BigDecimal.ZERO);
                        BigDecimal notDiscountClearanceFee = (prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice())).multiply(prodParam.getMinMeteringVolume()).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountClearanceFeeTotalPrice(notDiscountClearanceFee.subtract(prodParam.getClearanceFee()));
                    } else {
                        BigDecimal volume = prodParam.getVolume();
                        // 非空运才需要判断收费值计算清关费，空运以实际入仓值计算清关费
                        if (transportId == 1 || transportId == 2) {
                            if (Objects.nonNull(prodParam.getChargeVolume()) && prodParam.getChargeVolume().compareTo(BigDecimal.ZERO) > 0) {
                                volume = prodParam.getChargeVolume();
                            }
                            if (volume.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单体积大于0时，才比较最小收费计量
                                volume = (volume.compareTo(prodParam.getMinMeteringVolume()) > 0 ?
                                        volume : prodParam.getMinMeteringVolume()).setScale(2, RoundingMode.HALF_UP);
                            }
                        }
                        prodParam.setClearanceFee(prodParam.getOneClearanceFee()
                                .multiply(volume).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountClearanceFee = (prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice())).multiply(volume).setScale(2, RoundingMode.HALF_UP);
                        prodParam.setDiscountClearanceFeeTotalPrice(notDiscountClearanceFee.subtract(prodParam.getClearanceFee()));
                    }
                } else if (StringUtils.equals("kg", unitDO.getFuhao())) {
                    if (null == prodParam.getWeight() || prodParam.getWeight().compareTo(BigDecimal.ZERO) <= 0) {
                        prodParam.setClearanceFee(BigDecimal.ZERO);
                        BigDecimal notDiscountClearanceFee = (prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice())).multiply(prodParam.getMinMeteringWeight()).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountClearanceFeeTotalPrice(notDiscountClearanceFee.subtract(prodParam.getClearanceFee()));
                    } else {
                        BigDecimal weight = prodParam.getWeight();
                        // 非空运才需要判断收费值计算清关费，空运以实际入仓值计算清关费
                        if (transportId == 1 || transportId == 2) {
                            if (Objects.nonNull(prodParam.getChargeWeight()) && prodParam.getChargeWeight().compareTo(BigDecimal.ZERO) > 0) {
                                weight = prodParam.getChargeWeight();
                            }
                            if (weight.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单重量大于0时，才比较最小收费计量
                                weight = (weight.compareTo(prodParam.getMinMeteringWeight()) > 0 ?
                                        weight : prodParam.getMinMeteringWeight()).setScale(2, RoundingMode.HALF_UP);
                            }
                        }
                        prodParam.setClearanceFee(prodParam.getOneClearanceFee()
                                .multiply(weight).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountClearanceFee = (prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice())).multiply(prodParam.getWeight()).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountClearanceFeeTotalPrice(notDiscountClearanceFee.subtract(prodParam.getClearanceFee()));
                    }
                } else if (StringUtils.equals("a", unitDO.getFuhao())) {
                    if (null == prodParam.getQuantity() || prodParam.getQuantity() <= 0) {
                        prodParam.setClearanceFee(BigDecimal.ZERO);
                    } else {
                        // 数量计算
                        BigDecimal quantity = new BigDecimal(prodParam.getQuantity());
                        // 非空运才需要判断收费值计算清关费，空运以实际入仓值计算清关费
                        if (transportId == 1 || transportId == 2) {
                            if (Objects.nonNull(prodParam.getChargeQuantity()) && prodParam.getChargeQuantity().compareTo(BigDecimal.ZERO) > 0) {
                                quantity = prodParam.getChargeQuantity();
                            }
                            if (quantity.compareTo(BigDecimal.ZERO) > 0) {
                                // 当收费或填单数量大于0时，才比较最小收费计量
                                quantity = quantity.compareTo(prodParam.getMinMeteringQuantity()) > 0 ? quantity : prodParam.getMinMeteringQuantity();
                            }
                        }
                        prodParam.setClearanceFee(quantity.multiply(prodParam.getOneClearanceFee()).setScale(0, RoundingMode.HALF_UP));
                        BigDecimal notDiscountClearanceFee = (prodParam.getOneClearanceFee().add(prodParam.getDiscountClearanceFeePrice())).multiply(quantity).setScale(0, RoundingMode.HALF_UP);
                        prodParam.setDiscountClearanceFeeTotalPrice(notDiscountClearanceFee.subtract(prodParam.getClearanceFee()));
                    }
                } else {
                    throw exception(UNIT_NOT_LEGAL);
                }
            }
        }
    }


    /**
     * 商品询价
     *
     * @param departureId        始发地
     * @param objectiveId        目的地
     * @param prodConditionParam 商品信息
     * @return 根据始发地和目的地查询所有运输线路下的价格目录
     */
    public List<ProdFeeListDto> calculationProdFeeList(Long departureId, Long objectiveId, ProdConditionParam
            prodConditionParam) {
        if (null == prodConditionParam.getNum()) {
            prodConditionParam.setNum(1);
        }
        if (null == prodConditionParam.getQuantity()) {
            prodConditionParam.setQuantity(1);
        }

        if (null == prodConditionParam.getWeight()) {
            throw exception(WEIGHT_UNIT_NOT_NULL);
        }
        if (null == prodConditionParam.getVolume() || prodConditionParam.getVolume().compareTo(BigDecimal.ZERO) == 0) {
            if (StringUtils.isBlank(prodConditionParam.getBoxGauge())) {
                throw exception(PLEASE_ENTER_LENGTH_WIDTH_HEIGHT);
            } else {
                String[] box = prodConditionParam.getBoxGauge().split("\\*");
                if (box.length != 3) {
                    throw exception(PLEASE_ENTER_LENGTH_WIDTH_HEIGHT);
                }
                try {
                    BigDecimal volume = (new BigDecimal(box[0]).divide(new BigDecimal(100), 3, RoundingMode.HALF_UP))
                            .multiply(new BigDecimal(box[1]).divide(new BigDecimal(100), 3, RoundingMode.HALF_UP))
                            .multiply(new BigDecimal(box[2]).divide(new BigDecimal(100), 3, RoundingMode.HALF_UP));
                    prodConditionParam.setVolume(volume);
                } catch (Exception e) {
                    throw exception(PLEASE_ENTER_LENGTH_WIDTH_HEIGHT);
                }
            }
        }
        // 获取所有单位
        List<UnitDO> unitDOList = unitService.getUnitList();
        if (null == prodConditionParam.getFreightVolume() || null == prodConditionParam.getClearanceFeeVolume()) {
            // 默认使用体积单位
            UnitDO unitDO = unitDOList.stream().filter(u -> Objects.equals(u.getFuhao(), "m³")).findFirst().orElse(null);
            if (Objects.isNull(unitDO)) {
                throw exception(VOLUME_UNIT_NOT_NULL);
            }
            prodConditionParam.setFreightVolume(Long.valueOf(unitDO.getId()));
            prodConditionParam.setClearanceFeeVolume(Long.valueOf(unitDO.getId()));
        }

        Map<Integer, UnitDO> unitDOMap = unitDOList.stream()
                .collect(Collectors.toMap(UnitDO::getId, Function.identity(), (key1, key2) -> key2));
        List<ProdFeeListDto> prodFeeListDtoList = new ArrayList<>();
        // 获取始发地仓库信息
        List<WarehouseDO> startWarehouseList = warehouseService.findWarehouseListByCityIdAndTypes(departureId, Arrays.asList(2, 3));
        if (CollectionUtil.isEmpty(startWarehouseList)) {
            throw exception(DEPARTURE_NOT_START_WAREHOUSE);
        }
        List<WarehouseDO> destWarehouseList = warehouseService.findWarehouseListByCityIdAndTypes(objectiveId, Arrays.asList(1, 3));
        if (CollectionUtil.isEmpty(destWarehouseList)) {
            throw exception(OBJECTIVE_NOT_DEST_WAREHOUSE);
        }
        // 获取可用的运输方式列表
        List<DictDataRespDTO> transportDictList = dictDataApi.getDictDatas("transport_type");
        List<Integer> transportTypeList = transportDictList.stream().map(DictDataRespDTO::getValue).map(Integer::valueOf).collect(Collectors.toList());
        if (Objects.nonNull(prodConditionParam.getTransportId())) {
            if (transportTypeList.contains(prodConditionParam.getTransportId())) {
                // 选择运输方式时以选择的为准
                transportTypeList = Collections.singletonList(prodConditionParam.getTransportId());
            } else {
                // 当选择的运输方式不可用时直接返回空值
                return prodFeeListDtoList;
            }
        }
        List<LogisticsInfoDto> warehouseLineDOList = warehouseLineMapper.getStartInfoAndDestInfo(startWarehouseList,
                destWarehouseList, transportTypeList);
        if (CollectionUtil.isEmpty(warehouseLineDOList)) {
            throw exception(NOT_FIND_AVAILABLE_LINES);
        }
        Map<Long, LogisticsInfoDto> lineDOMap = warehouseLineDOList.stream().collect(Collectors.groupingBy(LogisticsInfoDto::getLineId,
                Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
        // 根据商品ID与线路ID获取商品运费与清关费的单价
        List<ProductPriceDO> productPriceDOList = productPriceService.list(new LambdaQueryWrapper<ProductPriceDO>()
                .eq(ProductPriceDO::getProductId, prodConditionParam.getProdId())
                .in(ProductPriceDO::getWarehouseLineId, warehouseLineDOList.stream().map(LogisticsInfoDto::getLineId).collect(Collectors.toList()))
                .eq(null != prodConditionParam.getChannelId(), ProductPriceDO::getShippingChannelId, prodConditionParam.getChannelId())
                .orderByDesc(ProductPriceDO::getId));

        if (CollectionUtil.isNotEmpty(productPriceDOList)) {
            for (ProductPriceDO productPriceDO : productPriceDOList) {
                ProdFeeListDto prodFeeListDto = new ProdFeeListDto();
                LogisticsInfoDto dto = lineDOMap.get(productPriceDO.getWarehouseLineId());
                prodFeeListDto.setTransportId(dto.getTransportId());

                prodFeeListDto.setStartWarehouseId(dto.getStartWarehouseId());
                prodFeeListDto.setStartTitleEn(dto.getStartTitleEn());
                prodFeeListDto.setStartTitleZh(dto.getStartTitleZh());
                prodFeeListDto.setDestWarehouseId(dto.getDestWarehouseId());
                prodFeeListDto.setDestTitleEn(dto.getDestTitleEn());
                prodFeeListDto.setDestTitleZh(dto.getDestTitleZh());
                prodFeeListDto.setChannelId(productPriceDO.getShippingChannelId());
                prodFeeListDto.setLineId(productPriceDO.getWarehouseLineId());
                prodFeeListDto.setCharging(productPriceDO.getPriceType());

                prodConditionParam.setOneFreight(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
                prodConditionParam.setFreightCurrency(Math.toIntExact(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPriceUnit() : productPriceDO.getAllPriceUnit()));
                prodConditionParam.setFreightVolume(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportVolumeUnit() : productPriceDO.getAllVolumeUnit());
                prodConditionParam.setCharging(productPriceDO.getPriceType());

                if (productPriceDO.getStepPrice() == 1) {
                    if (dto.getTransportId() == 1 || dto.getTransportId() == 2) {
                        this.stepSelect(unitDOMap, prodConditionParam, productPriceDO);
                    } else {
                        // 空运
                        BigDecimal quantity = BigDecimal.valueOf(prodConditionParam.getQuantity());
                        prodConditionParam.setTotalQuantity(quantity.compareTo(productPriceDO.getMinWeight()) > 0 ? quantity : productPriceDO.getMinWeight());
                        prodConditionParam.setTotalVolume(prodConditionParam.getVolume().compareTo(productPriceDO.getMinWeight()) > 0 ? prodConditionParam.getVolume() : productPriceDO.getMinWeight());
                        prodConditionParam.setTotalWeight(prodConditionParam.getWeight().compareTo(productPriceDO.getMinWeight()) > 0 ? prodConditionParam.getWeight() : productPriceDO.getMinWeight());
                        this.airStepSelect(unitDOMap, prodConditionParam, productPriceDO, dto.getLineId(), dto.getChannelId());
                    }
                }

                if (productPriceDO.getPriceType() == 0) {
                    if (dto.getTransportId() == 1 || dto.getTransportId() == 2) {
                        prodFeeListDto.setClearanceFreight(productPriceDO.getClearancePrice());
                        prodFeeListDto.setClearanceFreightCurrency(productPriceDO.getClearancePriceUnit());
                        prodFeeListDto.setClearanceFreightVolume(Math.toIntExact(productPriceDO.getClearanceVolumeUnit()));

                        prodConditionParam.setOneClearanceFee(productPriceDO.getClearancePrice());
                        prodConditionParam.setClearanceFeeCurrency(Math.toIntExact(productPriceDO.getClearancePriceUnit()));
                        prodConditionParam.setClearanceFeeVolume(productPriceDO.getClearanceVolumeUnit());
                    } else {
                        if (Objects.nonNull(productPriceDO.getShippingChannelId()) && productPriceDO.getShippingChannelId() > 0L) {
                            //空运清关费
                            AirClearancePriceResultDto airClearancePriceResultDto = productPriceService.queryAirClearancePrice(prodConditionParam.getProdId(), "", productPriceDO.getShippingChannelId(),
                                    prodConditionParam.getWeight(), prodConditionParam.getVolume(), Objects.isNull(prodConditionParam.getQuantity()) ? BigDecimal.ZERO : new BigDecimal(prodConditionParam.getQuantity()));

                            BasePrice basePrice = airClearancePriceResultDto.getClearancePrice();
                            log.info("空运渠道清关费：{}", Objects.nonNull(basePrice) ? JSONObject.toJSONString(basePrice) : null);
                            if (Objects.nonNull(basePrice)) {
                                prodFeeListDto.setClearanceFreight(basePrice.getClearancePrice());
                                prodFeeListDto.setClearanceFreightCurrency(basePrice.getClearancePriceUnit());
                                prodFeeListDto.setClearanceFreightVolume(Math.toIntExact(basePrice.getClearanceVolumeUnit()));

                                prodConditionParam.setOneClearanceFee(basePrice.getClearancePrice());
                                prodConditionParam.setClearanceFeeCurrency(Math.toIntExact(basePrice.getClearancePriceUnit()));
                                prodConditionParam.setClearanceFeeVolume(basePrice.getClearanceVolumeUnit());
                            } else {
                                prodFeeListDto.setClearanceFreight(basePrice.getClearancePrice());
                                prodFeeListDto.setClearanceFreightCurrency(basePrice.getClearancePriceUnit());
                                prodFeeListDto.setClearanceFreightVolume(Math.toIntExact(basePrice.getClearanceVolumeUnit()));

                                prodConditionParam.setOneClearanceFee(basePrice.getClearancePrice());
                                prodConditionParam.setClearanceFeeCurrency(Math.toIntExact(basePrice.getClearancePriceUnit()));
                                prodConditionParam.setClearanceFeeVolume(basePrice.getClearanceVolumeUnit());
                            }
                        } else {
                            prodFeeListDto.setClearanceFreight(BigDecimal.ZERO);
                            prodFeeListDto.setClearanceFreightCurrency(0L);
                            prodFeeListDto.setClearanceFreightVolume(0);

                            prodConditionParam.setOneClearanceFee(BigDecimal.ZERO);
                            prodConditionParam.setClearanceFeeCurrency(0);
                            prodConditionParam.setClearanceFeeVolume(0L);
                        }
                    }
                }

                prodFeeListDto.setSeaFreight(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
                prodFeeListDto.setSeaFreightCurrency(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPriceUnit() : productPriceDO.getAllPriceUnit());
                prodFeeListDto.setSeaFreightVolume(Math.toIntExact(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportVolumeUnit() : productPriceDO.getAllVolumeUnit()));
                prodFeeListDto.setSeaFreight(productPriceDO.getPriceType() == 0 ? productPriceDO.getTransportPrice() : productPriceDO.getAllPrice());
                // 获取路线商品特殊价格需求
                List<ProductPriceSpecialDO> specialDOList = specialService.list(new LambdaQueryWrapper<ProductPriceSpecialDO>()
                        .eq(ProductPriceSpecialDO::getDeleted, 0)
                        .eq(ProductPriceSpecialDO::getProductPriceId, productPriceDO.getId()));
                if (CollectionUtil.isNotEmpty(specialDOList)) {
                    Map<String, ProductPriceSpecialDO> specialDOMap = specialDOList.stream().collect(Collectors.groupingBy(ProductPriceSpecialDO::getSpecialDictType,
                            Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
                    // 计算有牌价格
                    if (null != prodConditionParam.getBrandType()) {
                        this.addBrandPrice(prodConditionParam, specialDOMap);
                    }
                    if (StringUtils.isNotBlank(prodConditionParam.getProdAttrIds())) {
                        // 获取该商品所有特性
                        List<ProductAttrDO> attrDOList = productAttrMapper.selectBatchIds(Arrays.asList(prodConditionParam.getProdAttrIds().split(StrUtil.COMMA)));
                        if (CollectionUtil.isNotEmpty(attrDOList)) {
                            Map<String, List<ProductAttrDO>> attrDOMap = attrDOList.stream().collect(Collectors.groupingBy(ProductAttrDO::getAttrCode));
                            // TODO 计算中性品牌价格
                            // 计算带电价格
                            this.addElectrifiedPrice(attrDOMap, prodConditionParam, specialDOMap);
                            // 计算液体价格
                            this.addLiquidPrice(attrDOMap, prodConditionParam, specialDOMap);
                        }
                    }
                }

                // 运费
                this.setTransportPrice(unitDOMap, prodConditionParam);

                if (0 == productPriceDO.getPriceType()) {
                    // 清关费
                    this.setClearancePrice(unitDOMap, prodConditionParam, prodFeeListDto.getTransportId());
                } else {
                    prodConditionParam.setClearanceFee(BigDecimal.ZERO);
                    prodConditionParam.setClearanceFee(BigDecimal.ZERO);
                    prodConditionParam.setClearanceFeeVolume(0L);
                    prodConditionParam.setClearanceFeeCurrency(0);
                }

                prodFeeListDto.setCharging(prodConditionParam.getCharging());
                if (prodFeeListDto.getCharging() == 0) {
                    prodFeeListDto.setEstClearanceFreight(prodConditionParam.getClearanceFee());
                    prodFeeListDto.setClearanceFreight(prodConditionParam.getOneClearanceFee());
                }
                prodFeeListDto.setEstSeaFreight(prodConditionParam.getFreight());
                prodFeeListDto.setSeaFreight(prodConditionParam.getOneFreight());
                prodFeeListDtoList.add(prodFeeListDto);
                warehouseLineDOList.remove(dto);
            }
        }

        for (LogisticsInfoDto logisticsInfoDto : warehouseLineDOList) {
            ProdFeeListDto prodFeeListDto = new ProdFeeListDto();
            prodFeeListDto.setTransportId(logisticsInfoDto.getTransportId());
            prodFeeListDto.setStartWarehouseId(logisticsInfoDto.getStartWarehouseId());
            prodFeeListDto.setStartTitleEn(logisticsInfoDto.getStartTitleEn());
            prodFeeListDto.setStartTitleZh(logisticsInfoDto.getStartTitleZh());
            prodFeeListDto.setDestWarehouseId(logisticsInfoDto.getDestWarehouseId());
            prodFeeListDto.setDestTitleEn(logisticsInfoDto.getDestTitleEn());
            prodFeeListDto.setDestTitleZh(logisticsInfoDto.getDestTitleZh());
            prodFeeListDto.setChannelId(prodConditionParam.getChannelId());
            prodFeeListDto.setLineId(logisticsInfoDto.getLineId());
            prodFeeListDto.setSeaFreight(BigDecimal.ZERO);
            prodFeeListDto.setEstSeaFreight(BigDecimal.ZERO);
            prodFeeListDto.setSeaFreightCurrency(0L);
            prodFeeListDto.setSeaFreightVolume(0);
            prodFeeListDto.setClearanceFreight(BigDecimal.ZERO);
            prodFeeListDto.setEstClearanceFreight(BigDecimal.ZERO);
            prodFeeListDto.setClearanceFreightCurrency(0L);
            prodFeeListDto.setClearanceFreightVolume(0);
            prodFeeListDto.setCharging(0);
            prodFeeListDtoList.add(prodFeeListDto);
        }

        return prodFeeListDtoList;

    }
}
