package cn.iocoder.yudao.module.product.param;

import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductPriceDO;
import cn.iocoder.yudao.module.product.dto.CouponAvailableDto;
import cn.iocoder.yudao.module.product.dto.CouponAvailableGroupDto;
import cn.iocoder.yudao.module.product.vo.coupon.CouponInfoVO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

@Data
@ApiModel("计价参数")
public class ProdConditionParam {


    @ApiModelProperty(value = "商品项id(报价单、订单的商品项id)")
    private Long itemId;

    @ApiModelProperty(value = "始发地")
    private Long departureId;

    @ApiModelProperty(value = "目的地")
    private Long objectiveId;

    @ApiModelProperty(value = "运输方式")
    private Integer transportId;

    @ApiModelProperty(value = "线路ID")
    private Long lineId;

    @ApiModelProperty(value = "出货渠道")
    private Long channelId;

    @ApiModelProperty(value = "发货联系人ID")
    private Long consignorCustomerContactsId;

    @ApiModelProperty(value = "收货联系人ID")
    private Long consigneeCustomerContactsId;

    @ApiModelProperty(value = "订单类型：1 普货 2 重货 3 泡货")
    private Integer orderType;

    @ApiModelProperty(value = "报关方式: 我司全代：1，自单代报：2，混合报关：3")
    private Integer customsType;

    @ApiModelProperty(value = "是否控货： 0 否  1 是")
    private Integer isControl;

    @ApiModelProperty(value = "商品ID")
    @NotNull(message = "商品ID不能为空")
    private Long prodId;

    @ApiModelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ApiModelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ApiModelProperty(value = "包装类型")
    private String packaging;

    @ApiModelProperty(value = "包装类型列表")
    private List<String> packages;

    @ApiModelProperty(value = "产品类型")
    private Integer productType;

    @ApiModelProperty(value = "商品体积")
    private BigDecimal volume = BigDecimal.ZERO;

    @ApiModelProperty(value = "商品重量")
    @NotNull(message = "商品重量不能为空")
    private BigDecimal weight = BigDecimal.ZERO;

    @ApiModelProperty(value = "重货体积")
    private BigDecimal wVolume;

    @ApiModelProperty(value = "泡货重量")
    private BigDecimal vWeight;

    @ApiModelProperty(value = "重货标准基数")
    private BigDecimal weightUnit;

    @ApiModelProperty(value = "泡货标准基数")
    private BigDecimal volumeUnit;

    @ApiModelProperty(value = "件数")
    private Integer num = 1;

    @ApiModelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ApiModelProperty(value = "数量")
    private Integer quantity = 0;

    @ApiModelProperty(value = "最小计量重量")
    private BigDecimal minMeteringWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "最小计量体积")
    private BigDecimal minMeteringVolume  = BigDecimal.ZERO;

    @ApiModelProperty(value = "最小计量个数")
    private BigDecimal minMeteringQuantity  = BigDecimal.ZERO;


    @ApiModelProperty(value = "收费重量")
    private BigDecimal chargeWeight;

    @ApiModelProperty(value = "收费体积")
    private BigDecimal chargeVolume ;

    @ApiModelProperty(value = "收费个数")
    private BigDecimal chargeQuantity;


    /**start** 空运阶梯价格匹配总值 **start**/

    @ApiModelProperty(value = "订单填单总重量")
    private BigDecimal totalWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单填单总体积")
    private BigDecimal totalVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单填单总个数")
    private BigDecimal totalQuantity = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单收费总重量")
    private BigDecimal chargeTotalWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单收费总体积")
    private BigDecimal chargeTotalVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单收费总个数")
    private BigDecimal chargeTotalQuantity = BigDecimal.ZERO;

    /**end** 空运阶梯价格匹配总值 **end**/

    @ApiModelProperty(value = "重货收费总体积")
    private BigDecimal heavyCargoTotalChargeVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "普货收费总体积")
    private BigDecimal commonGoodsTotalChargeVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "泡货收费总重量")
    private BigDecimal pickledGoodsTotalChargeWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "普货收费总重量")
    private BigDecimal commonGoodsTotalChargeWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "收费总个数")
    private BigDecimal totalChargeQuantity = BigDecimal.ZERO;

    @ApiModelProperty(value = "订单项属性：1 普货 2 重货 3 泡货")
    private Integer itemType;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    /**
     * 商品特性(可选多个特性ids)
     */
    private String prodAttrIds;

    /**
     * 已存在订单商品的同一天送货总方数
     */
    private BigDecimal alreadyExistsTotalVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "是否特价")
    private Boolean specialPriceType = false;

    @ApiModelProperty(value = "是否拆单自定义价格")
    private Boolean splitCustomPriceType = false;
    /**
     * 是否为渠道/单询动定价
     */
    @ApiModelProperty(value = "是否为渠道/单询手动定价")
    private Boolean channelManualPricing = false;

    @ApiModelProperty(value = "应收运费单价(原始价格(无牌无液无电)+特殊加价+明佣)")
    private BigDecimal receivableOneSeaFreight;

    @ApiModelProperty(value = "商品运费单价(成交)")
    private BigDecimal oneFreight;

    @ApiModelProperty(value = "商品运费单价(成交)-前端传值转换字段")
    private BigDecimal oneSeaFreight;

    @ApiModelProperty(value = "商品运费(小计)")
    private BigDecimal freight;

    @ApiModelProperty(value = "商品运费单价(原始单价)")
    private BigDecimal orgFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费金额(原始单价上进行优惠)")
    private BigDecimal discountFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费总金额")
    private BigDecimal discountFreightTotalPrice = BigDecimal.ZERO;


    @ApiModelProperty("包装单位")
    private String unit;

    @ApiModelProperty("入仓参数(此参数为前端实时计价接口参数，保持与订单入仓参数一致)")
    private WarehouseInInfoParam warehouseInInfoVO;
    /**
     * 运费用金额单位ID
     */
    private Integer freightCurrency;
    /**
     * 运费用金额单位ID
     */
    private Integer seaFreightCurrency;
    /**
     * 运费体积单位
     */
    private Long freightVolume;
    /**
     * 运费体积单位
     */
    private Long seaFreightVolume;

    @ApiModelProperty(value = "应收清关费单价(原始价格(无牌无液无电)+特殊加价)")
    private BigDecimal receivableOneClearanceFreight;
    @ApiModelProperty(value = "商品清关费单价(成交)")
    private BigDecimal oneClearanceFee;

    @ApiModelProperty(value = "商品清关费单价(成交)-前端传值转换字段")
    private BigDecimal oneClearanceFreight;

    @ApiModelProperty(value = "商品清关费(小计)")
    private BigDecimal clearanceFee;

    @ApiModelProperty(value = "商品清关费单价(原始)")
    private BigDecimal orgClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费金额(原始单价上进行优惠)")
    private BigDecimal discountClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费总金额")
    private BigDecimal discountClearanceFeeTotalPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "使用的优惠详情列表")
    private List<CouponInfoVO> couponInfoVOList;

    @ApiModelProperty(value = "是否已定价")
    private Boolean isPriced = Boolean.FALSE;

    @ApiModelProperty(value = "是否为改变收费数据的处理业务")
    private Boolean isChangeFeeValue  = Boolean.FALSE;
    /**
     * 清关费用货币单位ID
     */
    private Integer clearanceFeeCurrency;
    /**
     * 清关费用货币单位ID
     */
    private Integer clearanceFreightCurrency;
    /**
     * 清关费体积单位
     */
    private Long clearanceFeeVolume;
    /**
     * 清关费体积单位
     */
    private Long clearanceFreightVolume;

    @ApiModelProperty(value = "液体清关费单价")
    private BigDecimal liquidClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "液体运费单价")
    private BigDecimal liquidFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电清关费单价")
    private BigDecimal electrifiedClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电运费单价")
    private BigDecimal electrifiedFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌清关费单价")
    private BigDecimal brandClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌运费单价")
    private BigDecimal brandFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装清关费单价")
    private BigDecimal packagingClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装运费单价")
    private BigDecimal packagingFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "是否特价佣金设置")
    private Boolean specialCommission = Boolean.FALSE;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ApiModelProperty("明佣金额")
    private BigDecimal plainCommission = BigDecimal.ZERO;

    @ApiModelProperty("暗佣金额")
    private BigDecimal hiddenCommission = BigDecimal.ZERO;

    @ApiModelProperty(value = "无牌价0，有牌价1，中性品牌价2")
    private Integer brandType = 0;

    @ApiModelProperty(value = "品牌id")
    private Long brand;

    @ApiModelProperty(value = "箱规(如果没体积就以箱规计算：数据格式-长*宽*高)")
    private String boxGauge;

    @ApiModelProperty(value = "货值")
    private BigDecimal worth = BigDecimal.ZERO;

    @ApiModelProperty(value = "货值货币id")
    private Integer worthCurrency;

    @ApiModelProperty(value = "类型：1 正常计算 2 特价申请已传入的商品单价计算 3 指定重泡货标准计算 4 佣金设置 5 批量加价 6 订单优惠更新")
    private Integer calculationType = 1;

    @ApiModelProperty(value = "上次计价使用的优惠信息")
    private CouponAvailableGroupDto groupDto;

    @ApiModelProperty(value = "运费计量单位信息")
    private UnitDO unitDO;

    @ApiModelProperty(value = "清关费费计量单位信息")
    private UnitDO clearanceUnitDO;

    @ApiModelProperty(value = "可用优惠活动列表")
    private List<CouponAvailableDto> activityDtoList;

    @ApiModelProperty(value = "'空运加价运费单价'")
    private BigDecimal markupSeaFreight = BigDecimal.ZERO;

    @ApiModelProperty(value = "'空运加价清关费单价'")
    private BigDecimal markupClearanceFreight = BigDecimal.ZERO;
    @ApiModelProperty(value = "应收运费(成本) = 应收单价 * 原始重泡货标准的收费数据")
    private BigDecimal receivableSeaFreight;

    @ApiModelProperty(value = "应收清关费(成本) = 应收单价 * 原始重泡货标准的收费数据")
    private BigDecimal receivableClearanceFreight;

    @ApiModelProperty(value = "应收收费数量")
    private BigDecimal orgChargeQuantity = BigDecimal.ZERO;

    @ApiModelProperty(value = "应收收费件数")
    private Integer orgChargeNum = 0;

    @ApiModelProperty(value = "应收收费体积")
    private BigDecimal orgChargeVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "应收收费重量")
    private BigDecimal orgChargeWeight = BigDecimal.ZERO;


    @ApiModelProperty(value = "空运清关费来源：1 商品 2 渠道 3 商品线路价格 4 线路")
    private Integer airClearanceSource;

    @ApiModelProperty(value = "'空运清关费来源数据详情'")
    private String airClearanceInfo;


    @ApiModelProperty(value = "是否需要单询，0-否，1-是")
    private Integer needOrderInquiry;

    /**
     * 1-未设置商品清关费, 2-区间未匹配上
     */
    private int productPriceEmptyType = 0;

    /**
     * 1-未设置渠道清关费, 2-区间未匹配上, 3-包装类型未设置
     */
    private int channelPriceEmptyType = 0;

    /**
     * 是否超出空运重量上限
     */
    private Boolean isAirWeightLimit = false;

    private ProductPriceDO productPriceDO;


    public void setBrand(Long brand) {
        this.brand = brand;
    }


    public void setOneSeaFreight(BigDecimal oneSeaFreight) {
        this.oneSeaFreight = oneSeaFreight;
        this.oneFreight = oneSeaFreight;
    }

    public void setSeaFreightCurrency(Integer seaFreightCurrency) {
        this.seaFreightCurrency = seaFreightCurrency;
        this.freightCurrency = seaFreightCurrency;
    }

    public void setSeaFreightVolume(Long seaFreightVolume) {
        this.seaFreightVolume = seaFreightVolume;
        this.freightVolume = seaFreightVolume;
    }

    public void setClearanceFreightCurrency(Integer clearanceFreightCurrency) {
        this.clearanceFreightCurrency = clearanceFreightCurrency;
        this.clearanceFeeCurrency = clearanceFreightCurrency;
    }

    public void setClearanceFreightVolume(Long clearanceFreightVolume) {
        this.clearanceFreightVolume = clearanceFreightVolume;
        this.clearanceFeeVolume = clearanceFreightVolume;
    }

    public void setOneClearanceFreight(BigDecimal oneClearanceFreight) {
        this.oneClearanceFreight = oneClearanceFreight;
        this.oneClearanceFee = oneClearanceFreight;
    }
}
