package cn.iocoder.yudao.module.product.controller.admin.prod;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.module.product.component.ProdCostCalculation;
import cn.iocoder.yudao.module.product.dto.CalculationCostResultDto;
import cn.iocoder.yudao.module.product.param.ConditionParam;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.param.WarehouseInInfoParam;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.ORDER_ITEM_PROD_NOT_EXISTS;

@Api(tags = "管理后台 - 商品线路价格费用计算")
@RestController
@RequestMapping("/product/line/price")
@Validated
public class ProdLinePriceController {

    @Autowired
    private ProdCostCalculation prodCostCalculation;
    @Autowired
    private CouponService couponService;


    /**
     * 计算单种运输方式的商品费用
     *
     * @param param 条件计费参数
     * @return 商品列表计费后信息
     */
    @PostMapping("/calculation")
    @ApiOperation("计算单种运输方式的商品费用(单个商品也做数组传参)")
    public CommonResult<CalculationCostResultDto> calculationProdCost(@RequestBody ConditionParam param) {
        if (CollectionUtil.isEmpty(param.getProdConditionParamList())){
            return error(ORDER_ITEM_PROD_NOT_EXISTS);
        }
        // 前端实时计价时需要将订单的固定价格参数进行格式化，转为未定价，否则不会去查询当前实时价格
        param.setProdConditionParamList(param.getProdConditionParamList().stream().peek(it->{
            it.setIsPriced(false);
            WarehouseInInfoParam warehouseInInfoVO = it.getWarehouseInInfoVO();
            if (Objects.nonNull(warehouseInInfoVO)){
                it.setPackages(Arrays.asList(warehouseInInfoVO.getUnits().split(StrUtil.COMMA)));
                it.setVolume(warehouseInInfoVO.getVolume());
                it.setWeight(warehouseInInfoVO.getWeight());
                it.setQuantity(warehouseInInfoVO.getQuantityAll());
            }else {
                it.setPackages(Arrays.asList(it.getUnit().split(StrUtil.COMMA)));
            }
        }).collect(Collectors.toList()));
        List<ProdConditionParam> prodConditionParams = prodCostCalculation.obtainProdLineOnePrice(param.getProdConditionParamList(), "",
                Objects.nonNull(param.getCustomsType()) ? param.getCustomsType() : 1, param.getCustomerId(),
                param.getTransportId(), param.getLineId(), param.getChannelId(), false);
        CalculationCostResultDto dto = prodCostCalculation.calculationProdCost(prodConditionParams, "",
                param.getOrderType(), Objects.nonNull(param.getCustomsType()) ? param.getCustomsType() : 1, Objects.nonNull(param.getIsCargoControl()) && param.getIsCargoControl() ? 1 : 0,
                param.getUserId(), param.getCustomerId(), param.getConsignorCustomerId(), param.getConsigneeCustomerId(),
                param.getConsignorCustomerContactsId(), param.getConsigneeCustomerContactsId(), param.getTransportId(), param.getLineId(), param.getChannelId());
        return success(dto);
    }


}
